<?php

namespace App\Http\Controllers;

use App\Models\Company;
use App\Models\EmpCompanyDetails;
use App\Models\EmpPermit;
use App\Models\EmpTeam;
use App\Models\EmpTeamsMember;
use App\Models\EmpType;
use App\Models\EmpWorkerType;
use App\Models\EmpWorkExperience;
use App\Models\Project;
use App\Models\Role;
use App\Models\RosterAssign;
use App\Models\RosterTemplate;
use App\Models\Sites;
use App\Models\Tier;
use App\Models\User;
use App\Models\Workspace;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class DemoController extends Controller
{
    public function createDemoCustomer($user)
    {
        $customer = User::find($user->id);
        $workspace = Workspace::where('customer_id', $customer->id)->first();
        
        try {
            DB::beginTransaction();
            
            $sites = $this->createDemoSites($customer->id, $workspace->id);
            
            $employees = $this->createDemoEmployees($customer->id, $workspace->id);
            
            $clients = $this->createDemoClients($customer->id, $workspace->id);
            
            $projects = $this->createDemoProjects($customer->id, $workspace->id, $clients);
            
            $this->createDemoRosters($customer->id, $workspace->id, $employees, $sites, $projects);
            
            DB::commit();
            
            return $this->success1(
                [
                    'customer' => $customer,
                    'workspace' => $workspace,
                    'sites' => $sites,
                    'employees' => $employees,
                    'clients' => $clients,
                    'projects' => $projects
                ], 
                'Demo customer created successfully with all related data'
            );
            
        } catch (\Exception $e) {
            DB::rollBack();
            return $this->message1('Error creating demo data: ' . $e->getmessage(), 500);
        }
    }


    private function createDemoSites($customerId, $workspaceId)
    {
        $sites = [];
        $siteNames = [
            'Headquarters',
            'Downtown Office',
            'Manufacturing Plant',
            'Distribution Center',
            'Research Facility'
        ];
        
        $addresses = [
            '123 Main Street, New York, NY 10001',
            '456 Market St, San Francisco, CA 94103',
            '789 Industrial Blvd, Chicago, IL 60607',
            '101 Logistics Way, Atlanta, GA 30303',
            '555 Innovation Dr, Boston, MA 02110'
        ];
        
        $coordinates = [
            ['latitude' => 40.7128, 'longitude' => -74.0060],  // New York
            ['latitude' => 37.7749, 'longitude' => -122.4194], // San Francisco
            ['latitude' => 41.8781, 'longitude' => -87.6298],  // Chicago
            ['latitude' => 33.7490, 'longitude' => -84.3880],  // Atlanta
            ['latitude' => 42.3601, 'longitude' => -71.0589]   // Boston
        ];
        
        $companyClients = Company::where('customer_id', $customerId)
                                ->where('workspace_id', $workspaceId)
                                ->take(5)
                                ->get()
                                ->pluck('id')
                                ->toArray();
        
        while (count($companyClients) < 5) {
            $companyClients[] = 1; 
        }
        
        $supervisors = EmpCompanyDetails::where('access_role', 'supervisor')
                          ->where('customer_id', $customerId)
                          ->where('workspace_id', $workspaceId)
                          ->take(5)
                          ->get()
                          ->pluck('id')
                          ->toArray();
        
        while (count($supervisors) < 5) {
            $supervisors[] = 1; 
        }
    
        foreach ($siteNames as $index => $siteName) {
            $streetAddress = explode(',', $addresses[$index])[0];
            
            $site = Sites::create([
                'title' => $siteName,
                'address' => $addresses[$index],
                'street_address' => $streetAddress,
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
                'latitude' => $coordinates[$index]['latitude'],
                'longitude' => $coordinates[$index]['longitude'],
                'area_radius' => rand(100, 500), 
                'company_client' => $companyClients[$index],
                'supervisor_id' => $supervisors[$index],
                'site_state' => rand(1, 3),
                'contact_person' => 'Contact Person ' . ($index + 1),
                'contact_email' => 'site' . ($index + 1) . '@example.com',
                'contact_phone' => '555-' . str_pad($index + 1, 3, '0', STR_PAD_LEFT),
                'created_by' => $customerId,
            ]);
            
            $sites[] = $site;
        }
        
        return $sites;
    }

    private function createDemoEmployees($customerId, $workspaceId)
    {
        $employees = [];
        $roles = Role::where('workspace_id', $workspaceId)->get();
        $workPermits = EmpPermit::where('workspace_id', $workspaceId)->get();
        $tier = Tier::where('workspace_id', $workspaceId)->first();
        
        if ($roles->isEmpty()) {
            $roles = [
                Role::create([
                    'title' => 'Manager',
                    'workspace_id' => $workspaceId,
                    'customer_id' => $customerId,
                    'code' => 'mgr',
                    'created_by' => $customerId,
                ]),
                Role::create([
                    'title' => 'Team Lead',
                    'workspace_id' => $workspaceId,
                    'customer_id' => $customerId,
                    'code' => 'tld',
                    'created_by' => $customerId,
                ])
            ];
        }
        
        if ($workPermits->isEmpty()) {
            $workPermits = [
                EmpPermit::create([
                    'title' => 'Standard Access',
                    'role_id' => $roles[0]->id,
                    'created_by' => $customerId,
                    'workspace_id' => $workspaceId,
                    'customer_id' => $customerId,
                ])
            ];
        }
        
        $names = [
            'John Doe',
            'Jane Smith',
            'Robert Johnson',
            'Emily Wilson',
            'Michael Brown'
        ];
        
        foreach ($names as $index => $name) {
            $email = strtolower(str_replace(' ', '.', $name)) . '@example.com';
            
            $employee = EmpCompanyDetails::create([
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
                'employee_email' => $email,
                'employment_type' => ($index % 2) + 1, // Alternating between types
                'work_permit_type' => $workPermits[0]->id,
                'access_role' => $roles[$index % count($roles)]->id,
                'tier_id' => $tier->id,
                'start_date' => now()->subDays(rand(10, 60)),
                'end_date' => now()->addDays(rand(90, 180)),
                'plan_expire_date' => now()->addMonths(rand(3, 12)),
                'plan_effective_from' => now()->subDays(rand(10, 30)),
                'employment_start_date' => now()->subMonths(rand(1, 24))->toDateString(),
                'employment_end_date' => now()->addYears(rand(1, 5))->toDateString(),
                'completed' => 1,
                'status' => 1,
                'created_by' => $customerId,
                'updated_by' => $customerId,
                'employee_id' => 'EMP' . str_pad($index + 1, 4, '0', STR_PAD_LEFT),
                'job_title' => ['Manager', 'Developer', 'Designer', 'Assistant', 'Coordinator'][$index],
                'department' => ['HR', 'IT', 'Design', 'Operations', 'Marketing'][$index],
            ]);
            
            $employees[] = $employee;
        }
        
        $team = EmpTeam::create([
            'title' => 'Main Team',
            'customer_id' => $customerId,
            'workspace_id' => $workspaceId,
            'supervisor' => $employees[0]->id, 
            'description' => 'Main operational team',
            'created_by' => $customerId,
        ]);

        $workExperience = EmpWorkExperience::create([
            'title' => 'Game Development',
            'customer_id' => $customerId,
            'workspace_id' => $workspaceId,
            'created_by' => $customerId,
        ]);

        $workType = EmpWorkerType::create([
            'title' => 'Permanent',
            'customer_id' => $customerId,
            'workspace_id' => $workspaceId,
            'created_by' => $customerId,
        ]);
        
        $empType = EmpType::create([
            'title' => 'Part-time',
            'customer_id' => $customerId,
            'workspace_id' => $workspaceId,
            'created_by' => $customerId,
        ]);
        
        foreach ($employees as $employee) {
            EmpTeamsMember::create([
                'team_id' => $team->id,
                'emp_id' => $employee->id,
                'workspace_id' => $workspaceId,
                'customer_id' => $customerId,
                'created_by' => $customerId,
            ]);
        }
        
        return $employees;
    }

    private function createDemoClients($customerId, $workspaceId)
    {
        $clients = [];
        $demoClients = [
            [
                'first_name' => 'Michael',
                'middle_name' => '',
                'last_name' => 'Scott',
                'mobile' => '555-100',
                'phone' => '555-200',
                'email' => 'michael@acmecorp.com',
                'company_name' => 'Acme Corporation'
            ],
            [
                'first_name' => 'David',
                'middle_name' => 'R',
                'last_name' => 'Jones',
                'mobile' => '555-101',
                'phone' => '555-201',
                'email' => 'david@globaltech.com',
                'company_name' => 'GlobalTech Solutions'
            ],
            [
                'first_name' => 'Sarah',
                'middle_name' => 'J',
                'last_name' => 'Miller',
                'mobile' => '555-102',
                'phone' => '555-202',
                'email' => 'sarah@pinnacle.com',
                'company_name' => 'Pinnacle Industries'
            ],
            [
                'first_name' => 'Thomas',
                'middle_name' => '',
                'last_name' => 'White',
                'mobile' => '555-103',
                'phone' => '555-203',
                'email' => 'thomas@horizon.com',
                'company_name' => 'Horizon Services'
            ],
            [
                'first_name' => 'Jennifer',
                'middle_name' => 'L',
                'last_name' => 'Lee',
                'mobile' => '555-104',
                'phone' => '555-204',
                'email' => 'jennifer@elite.com',
                'company_name' => 'Elite Enterprises'
            ]
        ];
        
        foreach ($demoClients as $demoClient) {
            $client = Company::create([
                'fname' => $demoClient['first_name'],
                'mname' => $demoClient['middle_name'],
                'lname' => $demoClient['last_name'],
                'mobile' => $demoClient['mobile'],
                'phone' => $demoClient['phone'],
                'email' => $demoClient['email'],
                'logo_img' => 1,
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
                'created_by' => $customerId
            ]);
            
            $clients[] = $client;
        }
        
        return $clients;
    }

    private function createDemoProjects($customerId, $workspaceId, $clients)
    {
        $projects = [];
        $projectNames = [
            'Website Redesign',
            'Mobile App Development',
            'ERP Implementation',
            'Marketing Campaign',
            'Infrastructure Upgrade'
        ];
        
        $projectTypes = [
            'Web Development',
            'Mobile Development',
            'Software Implementation',
            'Marketing',
            'IT Infrastructure'
        ];
        
        foreach ($projectNames as $index => $projectName) {
            $client = $clients[$index];
            
            $project = Project::create([
                'title' => $projectName,
                'client_id' => $client->id,
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
                'description' => 'A project to ' . strtolower($projectName) . ' for ' . $client->name,
                'status' => ['active', 'pending', 'planning', 'active', 'pending'][$index],
                'type' => $projectTypes[$index],
                'start_date' => now()->subDays(rand(5, 30))->toDateString(),
                'end_date' => now()->addDays(rand(30, 90))->toDateString(),
                'budget' => rand(5000, 50000),
                'priority' => ['high', 'medium', 'low', 'high', 'medium'][$index],
                'created_by' => $customerId,
            ]);
            
            $projects[] = $project;
        }
        
        return $projects;
    }
    private function createDemoRosters($customerId, $workspaceId, $employees, $sites, $projects)
    {
        $shiftTypes = [
            'Morning' => ['start_time' => '08:00:00', 'end_time' => '16:00:00'],
            'Afternoon' => ['start_time' => '12:00:00', 'end_time' => '20:00:00'],
            'Night' => ['start_time' => '20:00:00', 'end_time' => '04:00:00'],
            'Standard' => ['start_time' => '09:00:00', 'end_time' => '17:00:00'],
            'Extended' => ['start_time' => '10:00:00', 'end_time' => '19:00:00']
        ];
    
        $rosterTemplates = [];
        foreach (array_keys($shiftTypes) as $shiftName) {
            $template = RosterTemplate::create([
                'start_time' => $shiftTypes[$shiftName]['start_time'],
                'end_time' => $shiftTypes[$shiftName]['end_time'],
                'color_code' => '#000000',  
                'break_minutes' => 30,      
                'shift_notes' => 'Demo shift for ' . $shiftName,
                'repeat_shift' => 'None',
                'repeat_every' => 'None',
                'end_date' => now()->addWeeks(4)->toDateString(),
                'is_saved' => 1,
                'created_by' => $customerId,
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
            ]);
    
            $rosterTemplates[$shiftName] = $template;
        }
    
        $startDate = now()->startOfWeek();
        $endDate = now()->addWeeks(4)->endOfWeek();
    
        foreach ($employees as $empIndex => $employee) {
            $site = $sites[$empIndex % count($sites)];
            $project = $projects[$empIndex % count($projects)];
    
            $currentDate = clone $startDate;
            while ($currentDate <= $endDate) {
                if ($currentDate->isWeekday()) {
                    $shiftTypeIndex = ($currentDate->dayOfWeek + $empIndex) % count($shiftTypes);
                    $shiftTypeName = array_keys($shiftTypes)[$shiftTypeIndex];
                    $shiftType = $shiftTypes[$shiftTypeName];
    
                    RosterAssign::create([
                        'customer_id' => $customerId,
                        'workspace_id' => $workspaceId,
                        'assign_to' => $employee->id,
                        'roster_template_id' => $rosterTemplates[$shiftTypeName]->id,
                        'schedule_date' => $currentDate->toDateString(),
                        'created_at' => now(),
                        'updated_at' => now(),
                    ]);
                }
    
                $currentDate->addDay();
            }
        }
    
        return true;
    }
    

    protected function success1($data, $message = 'Success')
    {
        return response()->json([
            'status' => true,
            'message' => $message,
            'data' => $data
        ], 200);
    }

    protected function message1($message, $code = 400)
    {
        return response()->json([
            'status' => false,
            'message' => $message
        ], $code);
    }

    protected function handleValidationFailure($validator)
    {
        return response()->json([
            'status' => false,
            'message' => 'Validation errors',
            'errors' => $validator->errors()
        ], 422);
    }


    protected function createSlug1($title)
    {
        return \Str::slug($title);
    }


}
