<?php

namespace App\Http\Controllers;

use App\Models\DynamicPages;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class PageController extends Controller
{
    public function index()
    {
        $customer_id = auth()->user()->id;
        $workspace_id = auth()->user()->current_workspace_id;
        $pages = DynamicPages::where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->get();
        return response()->json($pages);
    }

    // Store new page
    public function store(Request $request)
    {
        $validatedData = Validator::make($request->all(), [
            'title' => 'required|string|max:255',
            'permalink' => 'required|string|unique:pages',
            'main_heading' => 'required|string',
            'long_description' => 'required|string',
            'seo_title' => 'nullable|string',
            'seo_tags' => 'nullable|string',
            'seo_description' => 'nullable|string',
        ]);

        if ($validatedData->fails()) {
            return $this->error($validatedData->errors(), 422);
        }

        $validatedData = $validatedData->validated();
        $validatedData['customer_id'] = auth()->user()->id;
        $validatedData['workspace_id'] = auth()->user()->current_workspace_id;

        $page = DynamicPages::create($validatedData);

        return $this->success($page, 'Page Stored Successfully');
    }


    // Show single page
    public function show($id)
    {
        try {
            $page = DynamicPages::where('id', $id)
                ->when(config('auth.check_ownership'), function($query) {
                    return $query->where('customer_id', auth()->id())
                                 ->where('workspace_id', auth()->user()->current_workspace_id);
                })
                ->first();
    
            if (!$page) {
                return $this->error('Page not found or you don\'t have permission to view it', 404);
            }
    
            return $this->success(['page' => $page,], 'Page fetched successfully');
    
        } catch (\Exception $e) {
            \Log::error("Page fetch error [ID: $id]: " . $e->getMessage());
            return $this->error('Failed to retrieve page. Please try again.', 500);
        }
    }

    // Update page
    public function update(Request $request, $id)
    {
        $page = DynamicPages::findOrFail($id);

        $validatedData = Validator::make($request->all(), [
            'title' => 'sometimes|string|max:255',
            'permalink' => 'sometimes|string|unique:pages,permalink,' . $page->id,
            'main_heading' => 'sometimes|string',
            'long_description' => 'sometimes|string',
            'seo_title' => 'nullable|string',
            'seo_tags' => 'nullable|string',
            'seo_description' => 'nullable|string',
        ]);

        if ($validatedData->fails()) {
            return $this->error($validatedData->errors(), 422);
        }

        $validatedData = $validatedData->validated();

        $validatedData['customer_id'] = auth()->user()->id;
        $validatedData['workspace_id'] = auth()->user()->current_workspace_id;

        $page->update($validatedData);

        return $this->success($page, 'Page Updated Successfully');
    }


    // Delete page
    public function destroy($id)
    {
        try {
            $page = DynamicPages::find($id);
            
            if (!$page) {
                return $this->error('Page not found', 404);
            }
            
            if ($page->customer_id != auth()->user()->id || 
                $page->workspace_id != auth()->user()->current_workspace_id) {
                return $this->error('You don\'t have permission to delete this page', 403);
            }
            
            $page->delete();
            
            return $this->success($page, 'Page Deleted Successfully');
            
        } catch (\Exception $e) {
            \Log::error('Error deleting page: ' . $e->getMessage());
            return $this->error('Failed to delete page. Please try again.', 500);
        }
    }

    // Get page by permalink
    public function byPermalink($permalink)
    {
        try {
            $page = DynamicPages::where('permalink', urldecode($permalink))->first();
            
            if (!$page) {
                return $this->error('Page not found', 404);
            }
    
            return $this->success($page, 'Page fetched successfully');
            
        } catch (\Exception $e) {
            \Log::error("Page fetch error: " . $e->getMessage());
            return $this->error('Failed to retrieve page', 500);
        }
    }
}
