<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Traits\EmailTrait;
use App\Models\EmpCompanyDetails;
use App\Models\PasswordResetOtp;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class PasswordResetController extends Controller
{

    use EmailTrait;
    /**
     * Request OTP for password reset
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function requestOtp(Request $request)
{
    // Validate request
    $validator = Validator::make($request->all(), [
        'email' => 'required|email',
    ]);

    if ($validator->fails()) {
        Log::warning("OTP request validation failed", ['email' => $request->email]);
        return $this->handleValidationFailure($validator);
    }

    $email = $request->email;

    try {
        $user = User::where('email', $email)->first();

        if (!$user) {
            $user = EmpCompanyDetails::where('employee_email', $email)->first();
        }

        if (!$user) {
            Log::warning("OTP request failed - user not found", ['email' => $email]);
            return $this->message('User with this email does not exist', 404);
        }

        // Generate OTP
        $otp = PasswordResetOtp::generateOtp($email);
        Log::info("OTP generated", ['email' => $email, 'otp' => $otp]);

        // Prepare email content
        $emailParams = [
            'to' => $email,
            'subject' => 'Password Reset OTP',
            'msg' => view('Emails.otp-email', [
                'email' => $email,
                'otp_code' => $otp,
                'subject' => 'Password Reset OTP'
            ])->render()
        ];

        // Send OTP email
        $emailSent = $this->SendInstantEmail($emailParams);

        if (!$emailSent) {
            Log::error("Failed to send OTP email", ['email' => $email]);
            return $this->message('Failed to send OTP. Please try again later.', 500);
        }

        Log::info("OTP email sent successfully", ['email' => $email]);
        return $this->success($email, 'OTP sent to your email. It will expire in 5 minutes.');
    
    } catch (\Exception $e) {
        Log::critical("Error in requestOtp", ['email' => $email, 'error' => $e->getMessage()]);
        return $this->message('An unexpected error occurred. Please try again later.', 500);
    }
}

    /**
     * Verify OTP
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function verifyOtp(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'otp' => 'required|string|size:6',
        ]);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $email = $request->email;
        $otp = $request->otp;
        // Verify OTP
        $isValid = PasswordResetOtp::verifyOtp($email, $otp);
        if ($isValid) {
            // Generate a temporary token for password reset
            $token = Str::random(60);
            
            // Store token in the database
            PasswordResetOtp::updateOrCreate(
                ['email' => $email],
                ['token' => $token, 'created_at' => now()]
            );
            
            return $this->success($token, 'OTP verified successfully');
        } else {
            return $this->message('Invalid or expired OTP', 400);
        }
    }

    /**
     * Reset password
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function resetPassword(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'token' => 'required|string',
            'password' => 'required|string|min:8',
            'password_confirmation' => 'required|same:password'
        ]);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $email = $request->email;
        $token = $request->token;

        // Verify token from database
        $resetData = PasswordResetOtp::where('email', $email)
            ->where('token', $token)
            ->where('created_at', '>', now()->subHours(1))
            ->first();
            
        if (!$resetData) {
            return $this->message('Invalid or expired password reset token', 400);
        }
        
        // Update password
        try {
            $user = User::where('email', $email)->first();
            if (!$user) {
                $user = EmpCompanyDetails::where('employee_email', $email)->first();
            }
            if (!$user) {
                return $this->message('User with this email does not exist', 404);
            }

            $user->password = Hash::make($request->password);
            $user->remember_token = '';  // Clear any remember token
            $user->save();

            // Delete the used token
            $resetData->delete();

            return $this->message('Password reset successfully');
        } catch (\Exception $e) {
            Log::error('Failed to reset password: ' . $e->getMessage());
            return $this->message('An error occurred while resetting password', 500);
        }
    }
}
