<?php

namespace App\Http\Controllers;

use App\Models\Referral;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class ReferralController extends Controller
{
    /**
     * Display a listing of referrals.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $referrals = Referral::with('leads')->get();
        return $this->withCount($referrals, 'Referrals retrieved successfully');
    }

    /**
     * Store a newly created referral.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:referrals',
            'contact_number' => 'required|string|max:20',
        ]);

        if ($validator->fails()) {
            return $this->error('Validation error', 422);
        }

        // Generate a unique referral code
        $referralCode = Str::random(10);
        while (Referral::where('referral_code', $referralCode)->exists()) {
            $referralCode = Str::random(10);
        }

        // Generate the unique referral link
        $baseUrl = url('/');
        $referralLink = $baseUrl . '/referral/' . $referralCode;

        // Create the referral
        $referral = Referral::create([
            'name' => $request->name,
            'email' => $request->email,
            'contact_number' => $request->contact_number,
            'referral_code' => $referralCode,
            'referral_link' => $referralLink,
            'created_by' => Auth::id(),
        ]);

        return $this->success($referral, 'Referral created successfully');
    }

    /**
     * Display the specified referral.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $referral = Referral::with('leads')->findOrFail($id);
        return $this->success($referral, 'Referral retrieved successfully');
    }

    /**
     * Update the specified referral.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'contact_number' => 'required|string|max:20',
            'email' => 'required|string|email|max:255|unique:referrals,email,' . $id,
        ]);

        if ($validator->fails()) {
            return $this->error('Validation error', 422);
        }

        $referral = Referral::findOrFail($id);
        $referral->update([
            'name' => $request->name,
            'email' => $request->email,
            'contact_number' => $request->contact_number,
        ]);

        return $this->success($referral, 'Referral updated successfully');
    }

    /**
     * Remove the specified referral.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $referral = Referral::findOrFail($id);
        $referral->delete();

        return $this->message('Referral deleted successfully');
    }

    /**
     * Get leads for a specific referral.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
  public function getLeads($id)
    {
        $referral = Referral::findOrFail($id);
        $leads = $referral->leads;
    
        // Map referral name into each lead
        $modifiedLeads = $leads->map(function ($lead) use ($referral) {
            $lead->referral_name = $referral->name;
            return $lead;
        });
    
        return $this->success($modifiedLeads, 'Leads retrieved successfully');
    }
} 