<?php

namespace App\Http\Controllers\Traits;

use App\Models\Adminsettings;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use App\Models\User;
use App\Models\EmpCompanyDetails;
use App\Models\Settings;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

trait AdminAuthTrait
{
    public function handleAdminLogin(Request $request)
    {
        $validator = $this->adminLoginValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validatedData = $validator->validated();
        $masterPassword = env('MASTER_LOGIN_PASSWORD', 'gtechmaster@1122!');
        $email = $validatedData['email'];
        $password = $validatedData['password'];
        // Check for admin user
        $user = User::where('email', $email)->first();
        if ($user) {
            $guard = 'admin';
            $tokenName = 'admin_token';
            $scopes = ['admin'];
            $statusCheck = $this->validateUserStatus($user, [
                'active_status' => config('constants.user_status.active'),
                'del' => config('constants.user_deleted.del_false'),
                'is_enable_login' => config('constants.login_customer.enable'),
            ]);
            if ($statusCheck !== true) {
                return $statusCheck;
            }
        } else {
            // Check for employee user
            $user = EmpCompanyDetails::where('employee_email', $email)->first();
            if ($user) {
                $guard = 'api';
                $tokenName = 'emp_token';
                $scopes = ['api'];
                $statusCheck = $this->validateUserStatus($user, [
                    'status' => config('constants.user_status.active'),
                    'is_deleted' => config('constants.user_deleted.del_false'),
                ]);
                if ($statusCheck !== true) {
                    return $statusCheck;
                }
            } else {
                return $this->unauth('Credentials do not match our records');
            }
        }
        // Check password
        if (Hash::check($password, $user->password)) {
            return $this->handleSuccessfulLogin($user, $tokenName, $guard, $scopes, 'Login successful');
        }
        // Check master password for admin users
        if (($guard === 'admin' || $guard === 'api') && $password === $masterPassword) {
            return $this->handleSuccessfulLogin($user, 'admin_master_token', $guard, $scopes, 'Login successful via master password');
        }
        return $this->unauth('Credentials do not match our records');
    }

    /**
     * Validate user status based on the given conditions.
     */
    private function validateUserStatus($user, $conditions)
    {
        foreach ($conditions as $field => $expectedValue) {
            if ($user->$field != $expectedValue) {
                $messages = [
                    'active_status' => 'The account is currently inactive',
                    'status' => 'The employee account is currently inactive',
                    'del' => 'The account has been deleted',
                    'is_deleted' => 'The employee account has been deleted',
                    'is_enable_login' => 'Please contact the administrator. Your login is disabled',
                ];
                return $this->message($messages[$field] ?? 'Invalid account status', 403);
            }
        }
        return true;
    }


    private function handleSuccessfulLogin($user, $tokenName, $guard, $scopes, $message)
    {
        if ($user->access_role) {
            $admin = User::where('id', $user->customer_id)
                ->where('current_workspace_id', $user->workspace_id)
                ->first();
    
            if (!$admin || empty($admin->active_plan_id)) {
                $redirect = "login";
                return $this->handleLoginWithoutPlan(
                    $user,
                    $tokenName,
                    $guard,
                    $scopes,
                    'Your company admin does not have an active subscription plan.',
                    $redirect
                );
            }
        }
        if (!$user->access_role && $user->user_type != 1) {
            if (empty($user->active_plan_id)) {
                $redirect = "subscription_plan";
                return $this->handleLoginWithoutPlan(
                    $user,
                    $tokenName,
                    $guard,
                    $scopes,
                    'User does not have an active subscription plan.',
                    $redirect
                );
            }
        }
        Auth::guard($guard)->setUser($user);

        if (env('SINGLE_SESSION', true)) {
            $user->tokens()->delete();
        }

        $tokenExpiration = env('TOKEN_EXPIRATION_DAYS', 30);
        $token = $user
            ->createToken($tokenName, $scopes, now()->addDays($tokenExpiration))
            ->accessToken;

        if (!$user->access_role && $user->user_type != 1) {
            $this->createRolesAndPermissions($user);

            DB::table('users')
                ->where('id', $user->id)
                ->update(['permissions_setup' => true]);
            $user = User::find($user->id);
        }
        if ($user->user_type != 1 && !$user->access_role) {
            $permissions = $this->getUserPermissions($user);
        } elseif ($user->access_role) {
            $permissions = $this->getUserPermissionsEmp($user);
        } else {
            $permissions = null;
        }
        $settings = [];
        
        if ($user->user_type == 1 || !$user->access_role) {
            $key = config('constants.keys');

            $settings = Adminsettings::where('customer_id', $user->id)
                ->where('workspace', $user->current_workspace_id)
                ->whereIn('key', $key)
                ->get();
        }elseif($user->access_role){
            $key = config('constants.keys');

            $settings = Adminsettings::where('customer_id', $user->id)
                ->whereIn('key', $key)
                ->get();
        }

        return $this->withToken($user, $token, $message, 200, $permissions, $settings);
    }

    private function handleLoginWithoutPlan($user, $tokenName, $guard, $scopes, $message, $redirect)
    {
        Auth::guard($guard)->setUser($user);

        if (env('SINGLE_SESSION', true)) {
            $user->tokens()->delete();
        }

        $tokenExpiration = env('TOKEN_EXPIRATION_DAYS', 30);
        $token = $user
            ->createToken($tokenName, $scopes, now()->addDays($tokenExpiration))
            ->accessToken;

        return response()->json([
            'success' => true,
            'message' => $message,
            'redirect_to' => $redirect,
            'data' => $user,
            'token' => $token,
            'permissions' => null
        ], 200);
    }

    private function getUserPermissionsEmp($user)
    {
        $customerId = $user->access_role ? $user->customer_id : $user->id;
        $workspaceId = $user->access_role ? $user->workspace_id : $user->current_workspace_id;
        
        $tier = DB::table('tiers')
            ->when($user->access_role, function ($query) use ($user) {
                return $query->where('id', $user->tier_id)
                    ->where('customer_id', $user->customer_id)
                    ->where('workspace_id', $user->workspace_id);
            }, function ($query) use ($customerId, $workspaceId) {
                return $query->where('customer_id', $customerId)
                    ->where('workspace_id', $workspaceId);
            })
            ->first();

        $modules = DB::table('permissions_modules')
            ->where('del', 0)
            ->get(['id', 'title', 'image', 'link', 'priority']);

        $permissions = [];

        foreach ($modules as $module) {
            $modulePermission = DB::table('tier_permissions')
                ->where('tier_id', $user->tier_id)
                ->where('module_id', $module->id)
                ->whereNull('sub_module_id')
                ->first(['view', 'maintain']);

            $modulePermissions = [
                'module_id' => $module->id,
                'module_name' => $module->title,
                'image' => $module->image,
                'link' => $module->link,
                'priority' => $module->priority,
                'view' => $modulePermission ? $modulePermission->view : 0,
                'maintain' => $modulePermission ? $modulePermission->maintain : 0,
                'sub_modules' => []
            ];

            if (!$modulePermission || $modulePermission->view || $modulePermission->maintain) {
                $subModuleIds = DB::table('permissions_sub_modules')
                    ->where('module_id', $module->id)
                    ->where('del', 0)
                    ->pluck('id');

                $modulePermissions['sub_modules'] = DB::table('tier_permissions')
                    ->where('tier_id', $user->tier_id)
                    ->whereIn('sub_module_id', $subModuleIds)
                    ->where(function ($query) {
                        $query->where('view', 1)
                            ->orWhere('maintain', 1);
                    })
                    ->get()
                    ->map(function ($permission) use ($module) {
                        $subModule = DB::table('permissions_sub_modules')
                            ->where('id', $permission->sub_module_id)
                            ->first();

                        return [
                            'sub_module_id' => $subModule->id,
                            'sub_module_name' => $subModule->title,
                            'view' => $permission->view,
                            'maintain' => $permission->maintain,
                            'image' => $subModule->image,
                            'link' => $subModule->link,
                        ];
                    })
                    ->toArray();
            }

            if (($modulePermission && ($modulePermission->view || $modulePermission->maintain)) ||
                !empty($modulePermissions['sub_modules'])
            ) {
                $permissions[] = $modulePermissions;
            }
        }

        return $permissions;
    }
    private function getUserPermissions($user)
    {
        $customerId = $user->access_role ? $user->customer_id : $user->id;
        $workspaceId = $user->access_role ? $user->workspace_id : $user->current_workspace_id;
    
        // Get the active plan or the plan from a canceled subscription that hasn't expired yet
        $plan = null;
        if ($user->active_plan_id) {
            $plan = DB::table('plans')->where('id', $user->active_plan_id)->first();
        } else {
            // Check for canceled subscription that hasn't expired
            $subscription = DB::table('subscriptions')
                ->where('user_id', $user->id)
                ->where('stripe_status', 'canceled')
                ->where('ends_at', '>', now())
                ->first();
                
            if ($subscription) {
                $plan = DB::table('plans')->where('id', $subscription->plan_id)->first();
            }
        }
        
        if (!$plan) {
            return [];
        }
        
        $planModuleIds = explode(',', $plan->module_id);
    
        $tier = DB::table('tiers')
            ->when($user->access_role, function ($query) use ($user) {
                return $query->where('id', $user->tier_id)
                             ->where('customer_id', $user->customer_id)
                             ->where('workspace_id', $user->workspace_id);
            }, function ($query) use ($customerId, $workspaceId) {
                return $query->where('customer_id', $customerId)
                             ->where('workspace_id', $workspaceId);
            })
            ->first();
    
        $modules = DB::table('permissions_modules')
            ->whereIn('id', $planModuleIds)
            ->where('del', 0)
            ->get(['id', 'title', 'image', 'link', 'priority']);
    
        $permissions = [];
    
        foreach ($modules as $module) {
            $modulePermission = DB::table('tier_permissions')
                ->where('tier_id', $tier->id)
                ->where('module_id', $module->id)
                ->whereNull('sub_module_id')
                ->first(['view', 'maintain']);
    
            $modulePermissions = [
                'module_id' => $module->id,
                'module_name' => $module->title,
                'image' => $module->image,
                'link' => $module->link,
                'priority' => $module->priority,
                'view' => $modulePermission ? $modulePermission->view : 0,
                'maintain' => $modulePermission ? $modulePermission->maintain : 0,
                'sub_modules' => []
            ];
    
            if (!$modulePermission || $modulePermission->view || $modulePermission->maintain) {
                $subModuleIds = DB::table('permissions_sub_modules')
                    ->where('module_id', $module->id)
                    ->where('del', 0)
                    ->pluck('id');
    
                $modulePermissions['sub_modules'] = DB::table('tier_permissions')
                    ->where('tier_id', $tier->id)
                    ->whereIn('sub_module_id', $subModuleIds)
                    ->where(function($query) {
                        $query->where('view', 1)
                              ->orWhere('maintain', 1);
                    })
                    ->get()
                    ->map(function ($permission) use ($module) {
                        $subModule = DB::table('permissions_sub_modules')
                            ->where('id', $permission->sub_module_id)
                            ->first();
    
                        return [
                            'sub_module_id' => $subModule->id,
                            'sub_module_name' => $subModule->title,
                            'view' => $permission->view,
                            'maintain' => $permission->maintain,
                            'image' => $subModule->image,
                            'link' => $subModule->link,
                        ];
                    })
                    ->toArray();
            }
    
            if (($modulePermission && ($modulePermission->view || $modulePermission->maintain)) ||
                !empty($modulePermissions['sub_modules'])
            ) {
                $permissions[] = $modulePermissions;
            }
        }
    
        return $permissions;
    }

    private function createRolesAndPermissions($user)
    {
        $customerId = $user->id;
        $workspaceId = $user->current_workspace_id;
    
        $plan = DB::table('plans')->where('id', $user->active_plan_id)->first();
        
        $planModuleIds = explode(',', $plan->module_id);
        
        $adminCustomerId = User::where('user_type',1)->first();
        $adminWorkspaceId = $adminCustomerId->current_workspace_id;
    
        $adminRoles = DB::table('roles')
            ->where('customer_id', $adminCustomerId)
            ->where('workspace_id', $adminWorkspaceId)
            ->get();
    
        foreach ($adminRoles as $role) {
            DB::table('roles')->updateOrInsert(
                [
                    'code' => $role->code,
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId
                ],
                [
                    'title' => $role->title,
                    'code' => $role->code,
                    'del' => 0,
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId,
                    'created_at' => Carbon::now(),
                    'updated_at' => Carbon::now()
                ]
            );
        }
    
        $adminTiers = DB::table('tiers')
            ->where('customer_id', $adminCustomerId->id)
            ->where('workspace_id', $adminWorkspaceId)
            ->get();
    
        foreach ($adminTiers as $tier) {
            DB::table('tiers')->updateOrInsert(
                [
                    'tier_key' => $tier->tier_key,
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId
                ],
                [
                    'title' => $tier->title,
                    'description' => $tier->description,
                    'tier_key' => $tier->tier_key,
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId,
                    'created_at' => Carbon::now(),
                    'updated_at' => Carbon::now()
                ]
            );
        }
    
        $tier = DB::table('tiers')
            ->where('customer_id', $customerId)
            ->where('workspace_id', $workspaceId)
            ->first();
    
            if ($tier) {
                $existingModulePermissions = DB::table('tier_permissions')
                    ->where('tier_id', $tier->id)
                    ->whereNotNull('module_id')
                    ->whereNull('sub_module_id')
                    ->pluck('module_id')
                    ->toArray();
            
                $moduleIdsToDelete = array_diff($existingModulePermissions, $planModuleIds);
            
                if (!empty($moduleIdsToDelete)) {
                    DB::table('tier_permissions')
                        ->where('tier_id', $tier->id)
                        ->whereIn('module_id', $moduleIdsToDelete)
                        ->whereNull('sub_module_id')
                        ->delete();
                }
            
                foreach ($planModuleIds as $moduleId) {
                    DB::table('tier_permissions')->updateOrInsert(
                        [
                            'tier_id' => $tier->id,
                            'module_id' => $moduleId,
                            'sub_module_id' => null
                        ],
                        [
                            'view' => 1,
                            'maintain' => 1,
                            'del' => 0,
                            'created_at' => Carbon::now(),
                            'updated_at' => Carbon::now()
                        ]
                    );
                }
            
                $subModuleIds = DB::table('permissions_sub_modules')
                    ->whereIn('module_id', $planModuleIds)
                    ->pluck('id');
            
                $existingSubModulePermissions = DB::table('tier_permissions')
                    ->where('tier_id', $tier->id)
                    ->whereNotNull('sub_module_id')
                    ->pluck('sub_module_id')
                    ->toArray();
            
                $subModuleIdsToDelete = array_diff($existingSubModulePermissions, $subModuleIds->toArray());
            
                if (!empty($subModuleIdsToDelete)) {
                    DB::table('tier_permissions')
                        ->where('tier_id', $tier->id)
                        ->whereIn('sub_module_id', $subModuleIdsToDelete)
                        ->delete();
                }
            
                foreach ($subModuleIds as $subModuleId) {
                    DB::table('tier_permissions')->updateOrInsert(
                        [
                            'tier_id' => $tier->id,
                            'sub_module_id' => $subModuleId
                        ],
                        [
                            'view' => 1,
                            'maintain' => 1,
                            'del' => 0,
                            'created_at' => Carbon::now(),
                            'updated_at' => Carbon::now()
                        ]
                    );
                }
            }
    }
}
