<?php

namespace App\Http\Controllers\Traits;

use App\Models\Role;
use App\Models\Tier;
use App\Models\User;
use App\Models\EmpTeam;
use App\Models\EmpPermit;

use App\Models\Workspace;
use App\Models\PlanHistory;
use Illuminate\Http\Request;
use App\Models\EmpCompanyDetails;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use function PHPUnit\Framework\isEmpty;
use App\Http\Controllers\Traits\ValidationTrait;
use App\Models\Plan;
use App\Models\Subscription;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Traits\EmailTrait;
use App\Models\Adminsettings;

trait AdminCustomerTrait
{
    use ValidationTrait ,EmailTrait;

    public function signuphandle(Request $request)
    {
        // Validate the request data
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users,email',
            'password' => 'required|string|min:8|confirmed',
            'mobile_number' => 'nullable|string',
            'terms_accepted' => 'nullable|boolean',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }

        $validatedData = $validator->validated();

        try {
            DB::beginTransaction();

            // Create a workspace for the new customer
            $workspace = Workspace::create([
                'name' => 'Default Workspace',
                'status' => 1, // Active by default
                'slug' =>  'Default_Workspace',
                'is_disable' => 0, // Not disabled by default
                'created_by' => 1, // System created (will be updated after user creation)
            ]);

            // Save the original password for email
            $originalPassword = $validatedData['password'];
            
            // Create the user account
            $user = User::create([
                'name' => $validatedData['name'],
                'email' => $validatedData['email'],
                'mobile_number' => $validatedData['mobile_number'] ?? null,
                'password' => Hash::make($validatedData['password']),
                'is_enable_login' => 1, // Enable login by default for self-registered users
                'user_type' => config('constants.user_types.customer'),
                'active_status' => 1, // Active by default
                'current_workspace_id' => $workspace->id,
                'total_workspace' => 1,
                'total_active_workspace' => 1,
                'created_by' => 0,
            ]);

            // Update workspace with customer ID
            $workspace->update([
                'customer_id' => $user->id,
                'created_by' => $user->id,
            ]);

            // Generate token for API authentication
            $token = $user->createToken('auth_token');
            
            // Send welcome email
            $this->sendCustomerWelcomeEmail($user, 'Default Workspace', $originalPassword);

            DB::commit();

            return $this->success([
                'user' => $user,
                'token' => $token,
                'workspace' => $workspace
            ], 'Registration successful');
        } catch (\Exception $e) {
            DB::rollBack();
            return $this->message('Registration failed: ' . $e->getMessage(), 500);
        }
    }
    protected function handleCustomerCreate(Request $request)
    {
        $validator = $this->customerValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $user = User::where('email', $validationData['email'])->first();
        if ($user && $user->del != config('constants.user_deleted.del_false')) {
            return $this->message('This account has been deleted try some other email', 410);
        }
        if ($user && $user->active_status != config('constants.user_status.active')) {
            return $this->message('The account is currently inactive', 403);
        }
        if ($user) {
            return $this->message('Email already exists', 422);
        }
        try {
            DB::beginTransaction();
            $workspace = Workspace::create([
                'name' => $validationData['workspace_name'],
                'enable_domain' => $validationData['enable_domain'] ?? null,
                'domain_type' => $validationData['domain_type'] ?? null,
                'domain' => $validationData['domain'] ?? null,
                'subdomain' => $validationData['subdomain'] ?? null,
                'status' => $validationData['status'] ?? null,
                'slug' => $this->createSlug($validationData['workspace_name']) ?? null,
                'is_disable' => $validationData['is_disable'] ?? null,
                'created_by' => auth()->id() ?? 1, // Replace with authenticated user ID
            ]);
            $avatar = $this->handleFileImageUpload($request, 'UserAvatar');
            $avatar = $avatar['path'] ?? null;
            
            // Save the original password for email
            $originalPassword = $validationData['password'] ?? null;
            
            // Create Customer User
            $user = User::create([
                'name' => $validationData['customer_name'],
                'email' => $validationData['email'],
                'mobile_number' => $validationData['mobile_number'] ?? null,
                'password' => isset($validationData['password']) ? Hash::make($validationData['password']) : null,
                'is_enable_login' => $validationData['is_enable_login'] ?? 0,
                'user_type' => $validationData['user_type'] ?? config('constants.user_types.customer'),
                'active_status' => $validationData['active_status'] ?? 1,
                'current_workspace_id' => $workspace->id,
                'total_workspace' => $validationData['total_workspace'] ?? 1,
                'total_active_workspace' => $validationData['total_active_workspace'] ?? 1,
                'avatar' => $avatar,
                'active_plan_id' => $validationData['active_plan_id'] ?? null,
                'plan_expire_date' => $validationData['plan_expire_date'] ?? null,
                'created_by' => auth()->id() ?? 1, // Replace with authenticated user ID
            ]);
            // Update Workspace with Customer ID
            $workspace->update(['customer_id' => $user->id]);


            // 6) Insert a new Role in the roles table
            $role = Role::create([
                'title'        => 'Supervisor',   // Example title
                'workspace_id' => $workspace->id, // Link to the same workspace
                'customer_id'  => $user->id,      // Add customer id to the role
                'code'         => 'spv',          // Example code
                'created_by'   => $user->id ?? 1,
            ]);

            // 7) Insert a new record in the emp_permits table referencing the new Role
            // dd($role->id,$user->id,$workspace->id);
            $workpermit =  EmpPermit::create([
                'title'        => 'Full HR Access', // Example title
                'role_id'      => $role->id,
                'created_by'   => $user->id,    // Link to the Role
                'workspace_id' => $workspace->id,
                'customer_id'  => $user->id,       // Add customer id here as well

            ]);

            // 8) Insert a new record in the tiers table
            $tier =  Tier::create([
                'title'        => 'Enterprise Tier',  // Example
                'code'         => 'ENT',              // Example
                'customer_id'  => $user->id,
                'workspace_id' => $workspace->id,
                'description'  => 'High-level tier with all features', // Example
            ]);

            $employee = EmpCompanyDetails::create([
                'customer_id'           => $user->id,
                'workspace_id'          => $workspace->id,

                // Using dummy email from user
                'employee_email'        => 'employee@example.com',
                'employment_type'         => 1,
                'work_permit_type'      => $workpermit->id,
                'access_role'      => $role->id,
                'tier_id'      => $tier->id,

                // Date fields with dummy values
                'start_date'            => now()->subDays(30),
                'end_date'              => now()->addDays(60),
                'plan_expire_date'      => now()->addMonth(),
                'plan_effective_from'   => now()->subDays(30),
                'employment_start_date' => now()->subYear()->toDateString(),
                'employment_end_date'   => now()->addYear()->toDateString(),

                'completed'             => 1,
                'status'             => 1,

                'created_by'            => 1,
                'updated_by'            => 1,
            ]);
            // 5) Insert into emp_teams table (Example)
            EmpTeam::create([
                'title'        => 'Admin Team',  // Example title
                'customer_id'  => $user->id,
                'workspace_id' => $workspace->id,
                'supervisor'   => $employee->id, // Example supervisor id
                'description'  => 'good', // Example description
                'created_by'   => $user->id ?? 1,
            ]);
            
            // Send welcome email
            $this->sendCustomerWelcomeEmail($user, $workspace->name, $originalPassword);

            DB::commit();
            return $this->success($user, 'Customer created successfully');
        } catch (\Exception $e) {
            DB::rollBack();
            return $this->message('An error occurred: ' . $e->getMessage(), 500);
        }
    }

    /**
     * Send welcome email to newly created customers
     *
     * @param User $user
     * @param string $workspaceName
     * @param string|null $password
     * @return void
     */
    protected function sendCustomerWelcomeEmail($user, $workspaceName, $password = null)
    {
        try {
            $appUrl = config('app.url');
            $loginUrl = $appUrl . '/login';
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Welcome to ' . env('APP_NAME'),
                'msg' => view('Emails.customer-welcome', [
                    'subject' => 'Welcome to ' . env('APP_NAME'),
                    'name' => $user->name,
                    'email' => $user->email,
                    'workspace_name' => $workspaceName,
                    'password' => $password,
                    'login_url' => $loginUrl ?? null
                ])->render()
            ];
            
            $this->SendInstantEmail($emailParams);
            Log::info('Welcome email sent to new customer: ' . $user->email);
        } catch (\Exception $e) {
            Log::error('Error sending welcome email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }

    protected function handleAdminResetPassword($request)
    {
        $validator = $this->adminResetPasswordValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $user = User::findOrFail($request->id);
        
        // Store the original password for the email
        $originalPassword = $validationData['password'];
        
        // Hash and save the password
        $user->password = Hash::make($validationData['password']);
        $user->save();
        
        // Send password reset email
        $this->sendPasswordResetEmail($user, $originalPassword);
        
        return $this->message('Password reset successfully and notification sent to user.');
    }

    /**
     * Send password reset notification email
     *
     * @param User $user
     * @param string $password
     * @return void
     */
    protected function sendPasswordResetEmail($user, $password)
    {
        try {
            $appUrl = config('app.url');
            $loginUrl = $appUrl . '/login';
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Your Password Has Been Reset - ' . env('APP_NAME'),
                'msg' => view('Emails.password-reset', [
                    'subject' => 'Your Password Has Been Reset',
                    'name' => $user->name,
                    'email' => $user->email,
                    'password' => $password,
                    'login_url' => $loginUrl ?? null
                ])->render()
            ];
            
            $this->SendInstantEmail($emailParams);
            Log::info('Password reset email sent to user: ' . $user->email);
        } catch (\Exception $e) {
            Log::error('Error sending password reset email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }

    protected function handleCustomerShow($request)
    {
        $customers = User::with('plan')->where('user_type', config('constants.user_types.customer'))
            ->where('del', 0);
        if (!$customers->exists()) {
            return $this->message("No customers found.", 404);
        }
        if (($request->all())) {
            return $this->searchFilter($customers, $request);
        }
        return $this->withCount($customers, "Customer list retrieved successfully.");
    }
    protected function handleCustomerProfile($id)
    {

        $userTable = $this->getUserTable();
        $customer = User::find($id);

        if (
            $userTable == "customer" && ($customer->current_workspace_id != auth()->user()->current_workspace_id || $customer->id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this User', 403);
        }
        if ($userTable == "emp" && ($customer->id != auth()->user()->customer_id || $customer->current_workspace_id != auth()->user()->workspace_id)) {

            return $this->message('You do not have access to this User', 403);
        }

        $customer = User::where('user_type', config('constants.user_types.customer'))
            ->where('id', $id)
            ->where('del', 0)
            ->first(); // or firstOrFail()

        if (!$customer) {
            return $this->message("No customers found.", 404);
        }

        return $this->success($customer, "Customer retrieved successfully.");
    }

    protected function handleAdminCustomerEdit($request)
    {
        $validator = $this->adminCustomerEditValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $user = User::find($request->id);
        if (!$user) {
            return $this->message('Customer not found', 404);
        }
        if (User::where('email', $request->email)->where('id', '!=', $user->id)->exists()) {
            return $this->message('Email already exists for another user', 400);
        }
        $user->update($validationData);
        return $this->success($user, 'Customer updated successfully');
    }


    protected function handleAdminCustomerDelete($id)
    {
        $user = User::find($id);
        if (!$user) {
            return $this->message('Customer not found', 404);
        }
        $user->update(['del' => 1]);
        return $this->success($user, 'Customer deleted successfully', 200);
    }
    public function handleToggleUserLogin($request)
    {
        $validator = $this->loginToggleValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $user = User::where('id', $request->customer_id)->first();
        if (!$user) {
            return $this->message("No customers found.", 404);
        }
        
        // Store previous login status before updating
        $previousStatus = $user->is_enable_login;
        
        // Update status
        $user->is_enable_login = $request->is_enable_login;
        $user->save();
        
        // Send email notification about account status change
        $this->sendAccountStatusEmail($user, $user->is_enable_login);
        
        return $this->success($user, "User login status has been updated successfully.");
    }
    
    /**
     * Send email notification when account status changes
     *
     * @param User $user
     * @param int $isEnabled
     * @return void
     */
    protected function sendAccountStatusEmail($user, $isEnabled)
    {
        try {
            $appUrl = config('app.url');
            $loginUrl = $appUrl . '/login';
            
            $status = $isEnabled ? 'enabled' : 'disabled';
            $subject = 'Your Account Has Been ' . ucfirst($status);
            
            $emailParams = [
                'to' => $user->email,
                'subject' => $subject . ' - ' . env('APP_NAME'),
                'msg' => view('Emails.account-status', [
                    'subject' => $subject,
                    'name' => $user->name,
                    'email' => $user->email,
                    'status' => $status,
                    'login_url' => $isEnabled ? $loginUrl : null
                ])->render()
            ];
            
            $this->SendInstantEmail($emailParams);
            Log::info('Account status email sent to user: ' . $user->email . ' (Status: ' . $status . ')');
        } catch (\Exception $e) {
            Log::error('Error sending account status email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }

    public function handleCompanyLogin(Request $request)
    {
        $validator = $this->companyLoginValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validatedData = $validator->validated();
        $masterPassword = env('MASTER_LOGIN_PASSWORD', 'gtechmaster@1122!');
        $user = User::where("id", $validatedData["customer_id"])->first();
        if (!$user) {
            return $this->message('Something went wrong. Please try again later.');
        }
        if ($user && $user->user_type === config('constants.user_types.customer')) {
            // Get active subscription if exists
            $subscription = Subscription::where('user_id', $user->id)
                ->where(function ($query) {
                    $query->where('stripe_status', 'active')
                        ->orWhere('stripe_status', 'trialing')
                        ->orWhere(function ($q) {
                            $q->where('stripe_status', 'canceled')
                                ->where('ends_at', '>', now());
                        });
                })
                ->first();
        }
        $settings = null;
        if ($user->user_type == 1 || !$user->access_role) {
            $key = config('constants.keys');

            $settings = Adminsettings::where('customer_id', $user->id)
                ->where('workspace', $user->current_workspace_id)
                ->whereIn('key', $key)
                ->get();
        }
        $token = $user->createToken('admin_token', ['admin'])->accessToken;
        if(!$user->active_plan_id){
            return $this->handleLoginWithoutPlan($user, 'admin_token', 'admin', ['admin'], 'Login successful via master password' , 'subscription_plan' );
        }
        $this->createRolesAndPermissions($user);
        $permissions = $this->getUserPermissions($user);
        return $this->withCompanyTokenWithSubscription($user, $token, 1, $subscription, $permissions, $settings, 'Login successful via master password');
    }

    public function handleAssignPlan(Request $request)
    {
        $validator = $this->AssignPlanValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validatedData = $validator->validated();
        $user = User::findOrFail($validatedData['customer_id']);
        $currentDate = Carbon::now();
        // Expiry date after 30 days
        $expiryDate = $currentDate->copy()->addDays(30)->toDateString();

        $user->update([
            'active_plan_id' => $request->plan_id,
            'plan_expire_date' => $expiryDate,   // Use calculated expiry date
        ]);

        // Store the assignment history
        PlanHistory::create([
            'customer_id' => $request->customer_id,
            'plan_id' => $request->plan_id,
            'price' => $request->price,
            'type' => "Store",
            'assigned_at' => $currentDate,
            'expiration_date' => $expiryDate,  // Use calculated expiry date
        ]);
        if (!$user) {
            return $this->message('Something went wrong');  
        }
        
        $this->createRolesAndPermissions($user);
        $permissions = $this->getUserPermissions($user);

        $plan = Plan::where('id', $user->active_plan_id)->first();
        
        if ($plan->is_free_plan == 1) {
            $hasPreviousFreePlan = User::where('id', $user->id)
                ->where('free_plan', 1)
                ->exists();
            
            if ($hasPreviousFreePlan) {
                return $this->error('You have already used a free plan previously. Please choose a paid plan.', 422);
            }
        
            $user = User::find($user->id);
            
            if ($user->free_plan == 0) {
                $user->free_plan = 1;
                $user->save();
            }
        }
        
        // Send email notification about plan assignment
        $this->sendPlanAssignmentEmail($user, $plan, $expiryDate);
        
        return $this->message('Plan has been successfully assigned to customer');
    }
    
    /**
     * Send email notification when a plan is assigned to a user
     *
     * @param User $user
     * @param Plan $plan
     * @param string $expiryDate
     * @return void
     */
    protected function sendPlanAssignmentEmail($user, $plan, $expiryDate)
    {
        try {
            $appUrl = config('app.url');
            $loginUrl = $appUrl . '/login';
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Plan Assigned to Your Account - ' . env('APP_NAME'),
                'msg' => view('Emails.plan-assigned', [
                    'subject' => 'Plan Assigned to Your Account',
                    'name' => $user->name,
                    'plan_name' => $plan->name,
                    'price' => $plan->package_price_monthly,
                    'currency' => $plan->currency ?? 'USD',
                    'expiry_date' => $expiryDate,
                    'is_free_plan' => $plan->is_free_plan,
                    'login_url' => $loginUrl ?? null
                ])->render()
            ];
            
            $this->SendInstantEmail($emailParams);
            Log::info('Plan assignment email sent to user: ' . $user->email);
        } catch (\Exception $e) {
            Log::error('Error sending plan assignment email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }
}
