<?php

namespace App\Http\Controllers\Traits;

use App\Models\Adminsettings;
use App\Models\Settings;
use App\Models\User;
use SendGrid;
use SendGrid\Mail\Mail;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;

trait EmailTrait
{
    /**
     * Get email settings from database
     * 
     * @return array
     */
    private function getEmailSettings()
    {
        try {
            $emailOptions = [
                'email_mail_from_address',
                'email_mail_encryption', 
                'email_api_key',
                'email_mail_from_name', 
                'email_mail_host', 
                'email_mail_port', 
                'email_mail_password'
            ];
            $admin = User::where('user_type', config('constants.user_types.admin'))->first();
            $settings = Adminsettings::where('customer_id', $admin->id)
                ->whereIn('key', $emailOptions)
                ->pluck('value', 'key')
                ->mapWithKeys(function ($value, $key) {
                    $key = $key;
                    return [$key => $value];
                })
                ->toArray();
    
            return $settings;
        } catch (\Exception $e) {
            Log::error("Failed to fetch email settings: " . $e->getMessage());
            return [];
        }
    }

    /**
     * Send email using customer specific email configuration
     * 
     * @param array $params
     * @return bool
     */
    public function SendInstantEmail(array $params)
    {
        try {
            // Validate email address
            if (empty($params['to']) || !filter_var($params['to'], FILTER_VALIDATE_EMAIL)) {
                Log::error("Email sending failed: Invalid recipient email", [
                    'to' => $params['to'] ?? 'not provided'
                ]);
                return false;
            }
            
            // Get email settings from database
            $emailSettings = $this->getEmailSettings();
            
            // Validate sender email
            if (empty($emailSettings['email_mail_from_address']) || 
                !filter_var($emailSettings['email_mail_from_address'], FILTER_VALIDATE_EMAIL)) {
                Log::error("Email sending failed: Invalid sender email", [
                    'from' => $emailSettings['email_mail_from_address'] ?? 'not provided'
                ]);
                return false;
            }
            
            $email = new Mail();
            $email->setFrom(
                $emailSettings['email_mail_from_address'] , 
                $emailSettings['email_mail_from_name'] ?? "WMS"
            );
            $email->setSubject($params['subject']);
            $email->addTo($params['to']);
            $email->addContent("text/plain", strip_tags($params['msg'])); // Plain text version
            $email->addContent("text/html", $params['msg']); // HTML version
            
            // Use API key from settings table or fall back to .env
            $password = $emailSettings['email_mail_password'] ?? null;
            if (empty($password)) {
                Log::error("Email sending failed: No API key found");
                return false;
            }
            
            $sendgrid = new SendGrid($password);
            $response = $sendgrid->send($email);

            if ($response->statusCode() >= 200 && $response->statusCode() < 300) {
                return true;
            } else {
                Log::error("SendGrid email failed. Status Code: " . $response->statusCode());
                return false;
            }
        } catch (\Exception $e) {
            Log::error("Email sending failed: " . $e->getMessage());
            return false;
        }
    }
}