<?php

namespace App\Http\Controllers\Traits;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Str;




trait HelperTrait
{

    public function getLatestData($model, Request $request, $relations = [])
    {
        $latestData = $model->latest();
        if (!empty($relations)) {
            $relations = is_array($relations) ? $relations : [$relations];
            foreach ($relations as $relation) {
                $latestData = $latestData->with($relation);
            }
        }
        if (!empty($request->all())) {
            $latestData = $this->searchFilter($latestData, $request);
            return $this->success($latestData);
        }
        return $this->withCount($latestData);
    }

    public function handleFileImageUpload(Request $request, $directory = null, $currentFiles = null)
    {
        $directory = $directory ?? 'uploads'; // Default directory for uploads
        $uploadedPaths = [];
        $files = $request->allFiles(); // Get all uploaded files from the request

        foreach ($files as $key => $file) {
            if (!$file instanceof \Illuminate\Http\UploadedFile && !is_array($file)) {
                continue; // Skip non-file inputs
            }
            $fileItems = is_array($file) ? $file : [$file];
            foreach ($fileItems as $fileItem) {
                if (!$fileItem instanceof \Illuminate\Http\UploadedFile || !$fileItem->isValid()) {
                    continue;
                }
                // Get file extension and sanitize filename
                $extension = strtolower($fileItem->getClientOriginalExtension());
                $originalName = pathinfo($fileItem->getClientOriginalName(), PATHINFO_FILENAME);
                $sanitizedFileName = preg_replace('/[^a-zA-Z0-9_-]/', '', $originalName);
                $sanitizedFileName = substr($sanitizedFileName, 0, 100); // Limit filename length
                $uniqueFileName = time() . '_' . $sanitizedFileName . '.' . $extension;
                // Validate file type
                $allowedImageExtensions = ['jpg', 'jpeg', 'png', 'gif', 'webp'];
                $allowedDocumentExtensions = ['pdf', 'csv', 'doc', 'docx', 'xlsx', 'txt'];
                if (!in_array($extension, array_merge($allowedImageExtensions, $allowedDocumentExtensions))) {
                    continue; // Skip invalid file types
                }
                // Move file to public directory
                $fileItem->move(public_path($directory), $uniqueFileName);
                $uploadedPaths[] = [
                    'key' => $key, // Dynamic key
                    'path' => "$directory/$uniqueFileName",
                    'type' => in_array($extension, $allowedImageExtensions) ? 'image' : 'document'
                ];
                // Delete old file if it exists
                if (is_string($currentFiles) && File::exists(public_path($currentFiles))) {
                    File::delete(public_path($currentFiles));
                }
                if (is_array($currentFiles) && isset($currentFiles[$key])) {
                    foreach ((array) $currentFiles[$key] as $oldFilePath) {
                        if (File::exists(public_path($oldFilePath))) {
                            File::delete(public_path($oldFilePath));
                        }
                    }
                }
            }
        }
        return count($uploadedPaths) === 1 ? $uploadedPaths[0] : $uploadedPaths;
    }

    function createSlug($title)
    {
        return strtolower(preg_replace('/[^A-Za-z0-9-]+/', '-', trim($title)));
    }

    function generateTicketId()
    {
        do {
            $ticketId = 'TK-' . strtoupper(substr(uniqid(), -6));
        } while (\App\Models\HelpdeskTicket::where('ticket_id', $ticketId)->exists());
        return $ticketId;
    }

    function generateCsvFromQuery($query)
    {
        if ($query->getModel() === null) {
            return $this->message('Invalid query. Ensure the query is a valid Eloquent query.');
        }
        $modelName = strtolower(class_basename($query->getModel()));
        $directory = 'csv';
        $directoryPath = public_path($directory);
        if (!File::exists($directoryPath)) {
            File::makeDirectory($directoryPath, 0755, true);
        }
        $fileName = "{$modelName}.csv";
        $filePath = "{$directory}/{$fileName}";
        $fullFilePath = public_path($filePath);
        if (File::exists($fullFilePath)) {
            File::delete($fullFilePath);
        }
        $file = fopen($fullFilePath, 'w');
        $records = $query->get();
        if ($records->isEmpty()) {
            fclose($file);
            return $this->message('The query returned no records to export.');
        }
        $headers = array_keys($records->first()->toArray());
        fputcsv($file, $headers);
        foreach ($records as $record) {
            $flattenedRecord = array_map(function ($value) {
                if (is_array($value) || is_object($value)) {
                    return json_encode($value); // Convert to JSON string
                }
                return $value;
            }, $record->toArray());
            fputcsv($file, $flattenedRecord);
        }
        fclose($file);
        return asset($filePath);
    }
    function generateOrderId()
    {

        do {

            $OrderId = 'ODR-' . strtoupper(substr(uniqid(), -6));
        } while (\App\Models\Order::where('order_id', $OrderId)->exists());

        return $OrderId;
    }

    public static function generateCode($prefix = null)
    {
        $code = Str::upper(Str::random(10));
        if ($prefix) {
            return $prefix . $code;
        }
        return $code;
    }


    function getUserTable($user = null)
    {
        if (!$user) {
            $user = Auth::user();
        }
        if ($user instanceof \App\Models\User) {
            return 'customer';
        }
        if ($user instanceof \App\Models\EmpCompanyDetails) {
            return 'emp';
        }

        return null; // Return null if the user does not match either table
    }

    private function getPaymentStatusConfig($paymentStatus)
    {
        $configs = [
            // Pending - 0
            config('constants.payment_statuses.pending') => [
                'subject' => 'New Pending Payment Request',
                'status_text' => 'Pending',
                'color' => '#F59E0B', // Amber
                'background' => 'linear-gradient(135deg, #FEF3C7 0%, #FDE68A 100%)',
                'icon' => '⏳',
                'message' => 'A new payment request requires your attention and approval.',
                'action_required' => true,
                'action_message' => 'Please verify the payment and approve this request through the admin dashboard'
            ],
            // Completed - 1
            config('constants.payment_statuses.completed') => [
                'subject' => 'Payment Successfully Processed',
                'status_text' => 'Completed',
                'color' => '#22C55E', // Green
                'background' => 'linear-gradient(135deg, #D1FAE5 0%, #A7F3D0 100%)',
                'icon' => '✅',
                'message' => 'A payment has been successfully processed and confirmed.',
                'action_required' => false,
                'action_message' => ''
            ],
            // Failed - 2
            config('constants.payment_statuses.failed') => [
                'subject' => 'Payment Failed',
                'status_text' => 'Failed',
                'color' => '#EF4444', // Red
                'background' => 'linear-gradient(135deg, #FEE2E2 0%, #FECACA 100%)',
                'icon' => '❌',
                'message' => 'A payment attempt has failed and requires attention.',
                'action_required' => true,
                'action_message' => 'Please review the payment details and contact the customer if necessary'
            ],
            // Refunded - 3
            config('constants.payment_statuses.refunded') => [
                'subject' => 'Payment Refunded',
                'status_text' => 'Refunded',
                'color' => '#6366F1', // Indigo
                'background' => 'linear-gradient(135deg, #E0E7FF 0%, #C7D2FE 100%)',
                'icon' => '↩️',
                'message' => 'A payment has been refunded to the customer.',
                'action_required' => false,
                'action_message' => ''
            ],
            // Rejected - 4
            config('constants.payment_statuses.rejected') => [
                'subject' => 'Payment Rejected',
                'status_text' => 'Rejected',
                'color' => '#DC2626', // Dark Red
                'background' => 'linear-gradient(135deg, #FEE2E2 0%, #FCA5A5 100%)',
                'icon' => '⛔',
                'message' => 'A payment request has been rejected.',
                'action_required' => false,
                'action_message' => 'The payment has been reviewed and rejected'
            ]
        ];
        
        // Return the config for the requested status, or fallback to pending if status not found
        return $configs[$paymentStatus] ?? $configs[config('constants.payment_statuses.pending')];
    }


        /**
     * Get status configuration for email styling
     * 
     * @param int $status
     * @return array
     */
    private function getHelpdeskStatus($status)
    {
        // Define status configurations
        $statusConfigs = [
            0 => [ // In Progress
                'status_text' => 'In Progress',
                'color' => '#9333EA', // Purple
                'background' => 'linear-gradient(135deg, #F3E8FF 0%, #E9D5FF 100%)',
                'icon' => '🔄',
                'message' => 'Your ticket is currently being worked on.',
            ],
            1 => [ // Onhold
                'status_text' => 'On Hold',
                'color' => '#EA580C', // Orange
                'background' => 'linear-gradient(135deg, #FFEDD5 0%, #FED7AA 100%)',
                'icon' => '⏳',
                'message' => 'Your ticket is currently on hold.',
            ],
            2 => [ // Closed
                'status_text' => 'Closed',
                'color' => '#DC2626', // Slate
                'background' => 'linear-gradient(135deg, #FEE2E2 0%, #FCA5A5 100%)',
                'icon' => '📁',
                'message' => 'This ticket is now closed.',
            ],
        ];
        
        // Default status config if status not found
        $defaultConfig = [
            'status_text' => 'Unknown',
            'color' => '#6B7280', // Gray
            'background' => 'linear-gradient(135deg, #F3F4F6 0%, #E5E7EB 100%)',
            'icon' => '📄',
            'message' => 'Ticket status information.',
        ];
        
        return $statusConfigs[$status] ?? $defaultConfig;
    }

    /**
     * Get a descriptive payment type name based on payment type and status
     *
     * @param int $paymentType
     * @param int $paymentStatus
     * @return string
     */
    private function getPaymentTypeName($paymentType, $paymentStatus)
    {
        $type = 'Other';
        
        // Determine basic payment type
        if ($paymentType == config('constants.payment_types.bank')) {
            $type = 'Bank Transfer';
        } else if ($paymentType == config('constants.payment_types.stripe')) {
            $type = 'Stripe';
        } else if ($paymentType == config('constants.payment_types.pay_pal')) {
            $type = 'PayPal';
        }
        
        // Add status context for specific cases
        if ($paymentType == config('constants.payment_types.bank') && 
            $paymentStatus == config('constants.payment_statuses.pending')) {
            $type .= ' (Pending)';
        } else if ($paymentStatus == config('constants.payment_statuses.failed')) {
            $type .= ' (Failed)';
        } else if ($paymentStatus == config('constants.payment_statuses.refunded')) {
            $type .= ' (Refunded)';
        } else if ($paymentStatus == config('constants.payment_statuses.rejected')) {
            $type .= ' (Rejected)';
        }
        
        return $type;
    }
}
