<?php

namespace App\Http\Controllers\Traits;

use Illuminate\Http\Request;
use App\Models\Plan;
use App\Models\BankTransfer;
use App\Models\Order;
use App\Models\Subscription;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Traits\AdminAuthTrait;
use App\Models\AppSubscription;
use App\Http\Controllers\Traits\EmailTrait;
use App\Models\User;


trait PlanTrait
{
    use AdminAuthTrait;
    use EmailTrait;

    protected function storePlan($request){
        $validator = $this->subscriptionPlanValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $plan = Plan::create($validationData);
 
        return $this->success($plan ,'Plan created successfully');
    }
    protected function showPlan($plan){
        $plan = Plan::find($plan);
        if (!$plan) {
            return $this->message( 'Plan not found');
        }
        return $this->success($plan, 'Plan retrieved successfully');
    }
    protected function updatePlan($request){
        $validator = $this->subscriptionPlanValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $plan = Plan::find($request->id);
        if (!$plan) {
            return $this->message('Plan not found');
        }
        $plan->update($validationData);
        return $this->success($plan , 'Plan updated successfully');
    }
    protected function destroyPlan($plan){
        $plan = Plan::find($plan);
        if (!$plan) {
            return $this->message('Plan not found');
        }
        $plan->delete();
        return $this->message('Plan deleted successfully');
    }

    protected function customerSubscriptionRequest($request)
    {

        $validator = $this->customerSubscriptionRequestValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $validationData = $validator->validated();
        $image = $this->handleFileImageUpload($request, 'order');
        if ($image) {
            $validationData['receipt'] = $image['path'] ?? null;
        }
        // Check for existing active subscription
        $userId = Auth::id();
        $existingSubscription = Subscription::where('user_id', $userId)
            ->where(function($query) {
                $query->where('stripe_status', 'active')
                      ->orWhere('stripe_status', 'trialing');
            })
            ->first();
        // If there's an existing subscription, handle plan switching
        if ($existingSubscription) {
            try {
                // Cancel the existing subscription at period end
                $this->cancelSubscription($existingSubscription->stripe_id, true);
                // Update the existing subscription record
                $existingSubscription->ends_at = now()->addDays($existingSubscription->daysRemaining());
                $existingSubscription->save();
            } catch (\Exception $e) {
                Log::error('Error handling existing subscription: ' . $e->getMessage());
                return $this->error('Failed to process subscription change: ' . $e->getMessage(), 500);
            }
        }
        $orderId = $this->generateOrderId();
        $originalPrice = $validationData['price'] ?? 0;
        $finalPrice = $originalPrice;
        $couponData = [
            'coupon_applied' => false,
            'success' => true,
            'discount_amount' => 0,
            'coupon_code' => null,
            'coupon_id' => null,
            'message' => null,
            'final_price' => $originalPrice
        ];
        // Process coupon if provided
        if (isset($request->coupon_code) && !empty($request->coupon_code)) {
            $planIds = isset($validationData['plan_id']) ? [$validationData['plan_id']] : [];
            if (isset($request->planIds) && is_array($request->planIds)) {
                $planIds = $request->planIds;
            }
            $couponData = $this->processCouponFromRequest($request, $originalPrice, $planIds);
            if (!$couponData['success']) {
                return $this->error($couponData['message'], 422);
            }
            if ($couponData['coupon_applied']) {
                $finalPrice = $couponData['final_price'];
            }
        }
        // Check if this is a recurring subscription
        $isRecurring = false;
        if (isset($validationData['is_recurring'])) {
            if (
                $validationData['is_recurring'] === true ||
                $validationData['is_recurring'] === 1 ||
                $validationData['is_recurring'] === '1' ||
                strtolower($validationData['is_recurring']) === 'true'
            ) {
                $isRecurring = true;
            }
        }
        // Create order with updated price from coupon (if applied)
        $orderData = [
            'name' => $validationData['name'] ?? null,
            'email' => $validationData['email'] ?? null,
            'order_id' => $orderId,
            'plan_name' => $validationData['plan_name'] ?? null,
            'plan_id' => $validationData['plan_id'] ?? null,
            'is_recurring' => $isRecurring,
            'price' => $finalPrice,
            'original_price' => $originalPrice,
            'price_curency' => $validationData['price_curency'] ?? null,
            'payment_status' => $validationData['payment_status'] ?? 0,
            'receipt' => $validationData['receipt'] ?? null,
            'txn_id' => $validationData['txn_id'] ?? 0,
            'user_id' => $userId,
            'payment_type' => $validationData['payment_type'] ?? config('constants.payment_types.bank'),
            'interval' => $validationData['interval'] ?? null,
            'is_refund' => $validationData['is_refund'] ?? 0,
            'workspace' => Auth::user()->current_workspace_id ?? null,
        ];
        // Add coupon information to order if a coupon was applied
        if ($couponData['coupon_applied']) {
            $orderData['coupon_code'] = $couponData['coupon_code'];
            $orderData['discount_amount'] = $couponData['discount_amount'];
        }
        // Process stripe payment if the payment type is stripe
        if ($validationData['payment_type'] == config('constants.payment_types.stripe')) {
            $validationData['price'] = $finalPrice;
            $validationData['is_recurring'] = $isRecurring;
            $validationData['order_id'] = $orderId;
            // Get the plan information
            $plan = Plan::find($validationData['plan_id']);
            if ($plan) {
                $validationData['plan_name'] = $plan->name;
            }
            if ($isRecurring) {
                // Log the subscription creation attempt
                Log::info('Creating stripe subscription', [
                    'plan_id' => $validationData['plan_id'],
                    'amount' => $finalPrice,
                    'currency' => $validationData['price_curency'] ?? null
                ]);
                
                $stripeResult = $this->processStripeSubscription($request, $validationData);
            } else {
                // Log the one-time payment attempt
                Log::info('Processing one-time stripe payment', [
                    'plan_id' => $validationData['plan_id'],
                    'amount' => $finalPrice
                ]);
                
                $stripeResult = $this->processStripePayment($request, $validationData);
            }
            if ($stripeResult instanceof \Illuminate\Http\JsonResponse) {
                return $stripeResult;
            }
            $orderData['payment_status'] = config('constants.payment_statuses.completed');
            if ($isRecurring) {
                $orderData['stripe_subscription_id'] = $stripeResult['subscription']->id;
                
                // Fix for payment_intent access error
                if (isset($stripeResult['subscription']->latest_invoice) && 
                    isset($stripeResult['subscription']->latest_invoice->payment_intent) && 
                    isset($stripeResult['subscription']->latest_invoice->payment_intent->id)) {
                    $orderData['txn_id'] = $stripeResult['subscription']->latest_invoice->payment_intent->id;
                } else {
                    // Get the invoice ID as a fallback if payment_intent isn't available
                    $orderData['txn_id'] = $stripeResult['subscription']->latest_invoice->id ?? null;
                    Log::info('Using invoice ID as transaction ID. Payment intent not available.');
                }
                
                if (isset($stripeResult['price']->id)) {
                    $orderData['stripe_price_id'] = $stripeResult['price']->id;
                }
            } else {
                $orderData['txn_id'] = $stripeResult['charge']->id ?? null;
            }

            if (isset($stripeResult['customer']->id)) {
                $orderData['stripe_customer_id'] = $stripeResult['customer']->id;
            }
        }
        // Create the order in the database
        $order = Order::create($orderData);
        if (!$order) {
            return $this->error("Order creation failed", 500);
        }
        // Create subscription record if this is a recurring order
        if ($isRecurring && isset($stripeResult['subscription'])) {
            $subscription = $this->createSubscriptionRecord($order, $stripeResult);
            if (!$subscription) {
                return $this->error("Subscription record creation failed", 500);
            }
        }
        // Record coupon usage if coupon was applied
        if ($couponData['coupon_applied']) {
            $this->recordCouponUsage(
                $couponData['coupon_id'],
                $order->id,
                $couponData['discount_amount']
            );
        }
        
        // If payment type is bank transfer, send pending notification to customer
        if ($orderData['payment_type'] == config('constants.payment_types.bank')) {
            $this->sendBankTransferPendingEmail($order);
            
            // Send notification to admin
            $this->sendAdminPaymentNotification($order);
        }
        // If payment was processed, send a confirmation email
        else if ($orderData['payment_status'] == config('constants.payment_statuses.completed')) {
            if ($isRecurring) {
                $this->sendSubscriptionConfirmationEmail($order);
            } else {
                $this->sendPurchaseConfirmationEmail($order);
            }
            
            // Send notification to admin for completed payments too
            $this->sendAdminPaymentNotification($order);
        }
        
        // Update user's active plan and expiry date
        $user = Auth::user();
        if($validationData['payment_type'] != config('constants.payment_types.bank')){
            User::where('id', $user->id)->update(['active_plan_id' => $validationData['plan_id']]);
        }
        if ($isRecurring) {
            // For recurring subscriptions, set expiry date based on subscription period
            if (isset($stripeResult['subscription'])) {
                $currentPeriodEnd = Carbon::createFromTimestamp($stripeResult['subscription']->current_period_end);
                User::where('id', $user->id)->update(['plan_expire_date' => $currentPeriodEnd]);
                Log::info('Set recurring plan expiry date', [
                    'user_id' => $user->id,
                    'expiry_date' => $currentPeriodEnd->format('Y-m-d H:i:s')
                ]);
            } else {
                // Fallback for recurring with no subscription data
                $interval = $validationData['interval'] ?? 1;
                $expiryDate = now()->addMonths(is_numeric($interval) ? (int)$interval : 1);
                User::where('id', $user->id)->update(['plan_expire_date' => $expiryDate]);
                Log::info('Set fallback recurring plan expiry date', [
                    'user_id' => $user->id,
                    'expiry_date' => $expiryDate->format('Y-m-d H:i:s')
                ]);
            }
        } else {
            // For one-time purchases, set expiry date based on interval
            $expiryDate = null;
            
            // First check if we got the expiry date from Stripe
            if (isset($stripeResult['expiry_date']) && $stripeResult['expiry_date'] instanceof \Carbon\Carbon) {
                $expiryDate = $stripeResult['expiry_date'];
                Log::info('Using expiry date from Stripe result', [
                    'user_id' => $user->id, 
                    'expiry_date' => $expiryDate->format('Y-m-d H:i:s')
                ]);
            } 
            // If not, calculate based on interval
            else if (isset($validationData['interval'])) {
                $interval = $validationData['interval'];
                
                // If interval is not numeric, try to extract a numeric value
                if (!is_numeric($interval)) {
                    preg_match('/(\d+)/', $interval, $matches);
                    $interval = !empty($matches) ? (int)$matches[1] : 1;
                }
                
                $expiryDate = now()->addMonths((int)$interval);
                Log::info('Calculated expiry date from interval', [
                    'user_id' => $user->id,
                    'interval' => $interval,
                    'expiry_date' => $expiryDate->format('Y-m-d H:i:s')
                ]);
            } 
            // Fallback to 1 month
            else {
                $expiryDate = now()->addMonths(1);
                Log::info('Using fallback expiry date (1 month)', [
                    'user_id' => $user->id,
                    'expiry_date' => $expiryDate->format('Y-m-d H:i:s')
                ]);
            }
            
            // Update the user's expiry date
            User::where('id', $user->id)->update(['plan_expire_date' => $expiryDate]);
        }
                $plan = Plan::where('id', $request->plan_id)->first();

        if ($plan->is_free_plan == 1) {
            $userId = Auth::id();
            
            $hasPreviousFreePlan = User::where('id', $userId)
                ->where('free_plan', 1)
                ->exists();
            
            if ($hasPreviousFreePlan) {
                return $this->error('You have already used a free plan previously. Please choose a paid plan.', 422);
            }
            $user = User::find($userId);
            
            if ($user->free_plan == 0) {
                $user->free_plan = 1;
                $user->save();
            }
        }
        // Create roles and permissions
        $this->createRolesAndPermissions($user);
        $permissions = $this->getUserPermissions($user);
        if ($request->demo == true) {
            (new \App\Http\Controllers\DemoController)->createDemoCustomer($user);
        }             
        // Prepare response data
        $responseData = $order->toArray();
        if ($isRecurring && isset($subscription)) {
            $responseData['subscription'] = $subscription->toArray();
        }
        $responseData['permissions'] = $permissions;
        // Add coupon information to response if applied
        if ($couponData['coupon_applied']) {
            $responseData = array_merge(
                $responseData,
                [
                    'coupon' => [
                        'code' => $couponData['coupon_code'],
                        'discount_amount' => $couponData['discount_amount'],
                        'message' => $couponData['message']
                    ]
                ]
            );
        }
        $message = $isRecurring ? "Subscription created successfully" : "Order created successfully";
        if ($couponData['coupon_applied']) {
            $message .= " with coupon applied";
        }
        return $this->success($responseData, $message);
    }

    /**
     * Create a subscription record in the database
     *
     * @param Order $order
     * @param array $stripeResult
     * @return Subscription|null
     */
    protected function createSubscriptionRecord($order, $stripeResult)
    {
        try {
            $stripeSubscription = $stripeResult['subscription'];
            $stripeCustomer = $stripeResult['customer'];
            $stripeProduct = $stripeResult['product'];
            $stripePrice = $stripeResult['price'];

            // Log complete stripe subscription data for debugging
            Log::info('Processing stripe subscription data for record creation', [
                'subscription_id' => $stripeSubscription->id,
                'subscription_status' => $stripeSubscription->status,
                'has_latest_invoice' => isset($stripeSubscription->latest_invoice),
                'has_payment_intent' => isset($stripeSubscription->latest_invoice->payment_intent),
                'current_period_end' => $stripeSubscription->current_period_end ?? 'not available'
            ]);

            // Get the current period dates
            $currentPeriodStart = null;
            $currentPeriodEnd = null;
            
            if (isset($stripeSubscription->current_period_start) && $stripeSubscription->current_period_start > 0) {
                $currentPeriodStart = Carbon::createFromTimestamp($stripeSubscription->current_period_start);
            } else {
                $currentPeriodStart = now();
            }
            
            if (isset($stripeSubscription->current_period_end) && $stripeSubscription->current_period_end > 0) {
                $currentPeriodEnd = Carbon::createFromTimestamp($stripeSubscription->current_period_end);
            } else {
                // If current_period_end is not available, calculate it based on interval
                $interval = $order->interval ?? 1;
                $currentPeriodEnd = now()->addMonths($interval);
            }

            // Create or update subscription record
            $subscription = Subscription::updateOrCreate(
                ['stripe_id' => $stripeSubscription->id],
                [
                    'user_id' => $order->user_id,
                    'order_id' => $order->id,
                    'plan_id' => $order->plan_id,
                    'name' => $order->plan_name,
                    'stripe_status' => $stripeSubscription->status,
                    'stripe_price' => $stripePrice->id,
                    'stripe_product' => $stripeProduct->id,
                    'stripe_customer' => $stripeCustomer->id,
                    'quantity' => $stripeSubscription->items->data[0]->quantity ?? 1,
                    'current_period_start' => $currentPeriodStart,
                    'current_period_end' => $currentPeriodEnd,
                    'interval' => $order->interval,
                    'amount' => $order->price,
                    'currency' => $order->price_curency,
                    'is_recurring' => true,
                    'metadata' => [
                        'stripe_subscription_id' => $stripeSubscription->id,
                        'stripe_customer_id' => $stripeCustomer->id,
                        'stripe_price_id' => $stripePrice->id,
                        'stripe_product_id' => $stripeProduct->id,
                        'order_id' => $order->id,
                        'created_at' => now()->toIso8601String()
                    ]
                ]
            );

            // Update the order with subscription information
            $order->stripe_subscription_id = $stripeSubscription->id;
            $order->stripe_customer_id = $stripeCustomer->id;
            $order->stripe_price_id = $stripePrice->id;
            $order->save();

            // Log successful subscription creation
            Log::info('Created subscription record', [
                'subscription_id' => $subscription->id,
                'stripe_subscription_id' => $stripeSubscription->id,
                'user_id' => $order->user_id,
                'plan_id' => $order->plan_id
            ]);

            return $subscription;
        } catch (\Exception $e) {
            Log::error('Failed to create subscription record: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
            return null;
        }
    }

    /**
     * Send a subscription confirmation email
     *
     * @param Order $order
     * @return void
     */
    protected function sendSubscriptionConfirmationEmail($order)
    {
        try {
            $subscription = Subscription::where('order_id', $order->id)->first();
            if (!$subscription) {
                Log::error('Subscription not found for order: ' . $order->order_id);
                return;
            }
            
            $user = \App\Models\User::find($order->user_id);
            if (!$user) {
                Log::error('User not found for order: ' . $order->order_id);
                return;
            }
            
            // Determine next billing date with validation and fallbacks
            $nextBillingDate = null;
            
            // Try to get a valid next billing date from subscription
            if ($subscription->current_period_end) {
                try {
                    if ($subscription->current_period_end instanceof \Carbon\Carbon) {
                        if ($subscription->current_period_end->year > 1970) {
                            $nextBillingDate = $subscription->current_period_end;
                        }
                    } else {
                        // Parse the date string
                        $parsedDate = Carbon::parse($subscription->current_period_end);
                        if ($parsedDate->year > 1970) {
                            $nextBillingDate = $parsedDate;
                        }
                    }
                } catch (\Exception $e) {
                    Log::error('Error parsing next billing date: ' . $e->getMessage());
                    $nextBillingDate = null;
                }
            }
            
            // If next billing date couldn't be determined, calculate it
            if (!$nextBillingDate) {
                $interval = $order->interval ?? 1;
                
                // Ensure interval is numeric
                if (!is_numeric($interval)) {
                    preg_match('/(\d+)/', $interval, $matches);
                    $interval = !empty($matches) ? (int)$matches[1] : 1;
                }
                
                $nextBillingDate = now()->addMonths((int)$interval);
                
                // Update the subscription's current_period_end if it was invalid
                if ($subscription) {
                    Log::info('Updating invalid current_period_end for subscription: ' . $subscription->id);
                    $subscription->current_period_end = $nextBillingDate;
                    $subscription->save();
                }
            }
            
            // Format the date for the email
            $nextBillingDateFormatted = $nextBillingDate->format('Y-m-d');
            
            Log::info('Using next billing date in subscription confirmation email: ' . $nextBillingDateFormatted);
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Your Subscription has been Activated',
                'msg' => view('Emails.subscription-confirmation', [
                    'subject' => 'Your Subscription has been Activated',
                    'name' => $user->name ?? $order->name,
                    'plan_name' => $order->plan_name,
                    'order_id' => $order->order_id,
                    'price' => $order->price,
                    'currency' => $order->price_curency,
                    'interval' => $order->interval,
                    'next_billing_date' => $nextBillingDateFormatted
                ])->render()
            ];
            
            $emailSent = $this->SendInstantEmail($emailParams);
            if (!$emailSent) {
                Log::error('Failed to send subscription confirmation email for order: ' . $order->order_id);
                return $this->message('Failed to send confirmation email. Please try again later.', 500);
            }
            
            Log::info('Subscription confirmation email sent for order: ' . $order->order_id);
            
        } catch (\Exception $e) {
            Log::error('Error sending subscription confirmation email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }

    /**
     * Send a purchase confirmation email
     *
     * @param Order $order
     * @return void
     */
    protected function sendPurchaseConfirmationEmail($order)
    {
        try {
            $user = \App\Models\User::find($order->user_id);
            if (!$user) {
                Log::error('User not found for order: ' . $order->order_id);
                return;
            }
            
            // Determine expiry date with additional validation and fallbacks
            $expiryDate = null;
            
            // First check if user has a valid plan_expire_date
            if ($user->plan_expire_date) {
                try {
                    if ($user->plan_expire_date instanceof \Carbon\Carbon) {
                        if ($user->plan_expire_date->year > 1970) {
                            $expiryDate = $user->plan_expire_date;
                        }
                    } else {
                        // Parse the date string
                        $parsedDate = Carbon::parse($user->plan_expire_date);
                        if ($parsedDate->year > 1970) {
                            $expiryDate = $parsedDate;
                        }
                    }
                } catch (\Exception $e) {
                    Log::error('Error parsing plan expire date: ' . $e->getMessage());
                    $expiryDate = null;
                }
            }
            
            // If we couldn't get a valid date from the user model, calculate it from order data
            if (!$expiryDate) {
                $interval = $order->interval ?? 1;
                
                // Ensure interval is numeric
                if (!is_numeric($interval)) {
                    preg_match('/(\d+)/', $interval, $matches);
                    $interval = !empty($matches) ? (int)$matches[1] : 1;
                }
                
                $expiryDate = now()->addMonths((int)$interval);
                
                // Update the user's expiry date if it was invalid
                Log::info('Updating invalid plan expiry date for user: ' . $user->id . ' to ' . $expiryDate->format('Y-m-d'));
                User::where('id', $user->id)->update([
                    'plan_expire_date' => $expiryDate
                ]);
            }
            
            // Format the date for the email
            $expiryDateFormatted = $expiryDate->format('Y-m-d');
            
            Log::info('Using expiry date in purchase confirmation email: ' . $expiryDateFormatted);
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Your Purchase has been Confirmed',
                'msg' => view('Emails.purchase-confirmation', [
                    'subject' => 'Your Purchase has been Confirmed',
                    'name' => $user->name ?? $order->name,
                    'plan_name' => $order->plan_name,
                    'order_id' => $order->order_id,
                    'price' => $order->price,
                    'currency' => $order->price_curency,
                    'interval' => $order->interval,
                    'expiry_date' => $expiryDateFormatted
                ])->render()
            ];
            
            $emailSent = $this->SendInstantEmail($emailParams);
            if (!$emailSent) {
                Log::error('Failed to send purchase confirmation email for order: ' . $order->order_id);
            } else {
                Log::info('Purchase confirmation email sent for order: ' . $order->order_id);
            }
            
        } catch (\Exception $e) {
            Log::error('Error sending purchase confirmation email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }
    
    protected function showbankTransferRequestList()
    {
        $BankTransfer = Order::where('payment_type', config('constants.payment_types.bank'))->latest()->get();
        if ($BankTransfer->isEmpty()) {
            return $this->message('Bank Request not found');
        }
        return $this->withCount($BankTransfer, 'Bank Request retrieved successfully');
    }

    protected function handelShowRequestOrder()
    {
        $Order = Order::latest()->get();
        if ($Order->isEmpty()) {
            return $this->message('Bank Request not found');
        }
        return $this->withCount($Order, 'Bank Request retrieved successfully');
    }

    protected function handelbankTransferRequestDelete($id)
    {
        $Order = Order::find($id);
        if (!$Order) {
            return $this->message('Bank Transfer Request not found');
        }
        $Order->delete();
        return $this->message('Bank Transfer Request deleted successfully');
    }


    public function handelBankTransferRequestStatus($request)
    {
        $validator = $this->BankTransferRequestStatusValidationRequest($request);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        $Order = Order::find($request->bank_id);
        if (!$Order) {
            return $this->message('Bank Transfer Request not found');
        }
        $interval = $Order->interval;
        $plan_id =  $Order->plan_id;
        $price =  $Order->price;
        $Order->payment_status = $request->status;
        $Order->save();
        $currentDate = Carbon::now();
        $expiryDate  = $currentDate->addMonths($interval)->format('Y-m-d');
        $customer_id = $request->customer_id;
        // Send notification to admin about the status change
        $this->sendAdminPaymentNotification($Order);
        if ($request->status == 1) {
            // Send confirmation email based on whether this is a recurring subscription
      
            $request = new Request([
                'customer_id' => $customer_id,
                'plan_id' => $plan_id,
                'price' => $price,
                'expiration_date' => $expiryDate,
            ]);
            return $this->handleAssignPlan($request);
        } 
        // Send rejection notification when payment is rejected
        if ($request->status == 4) {
            $this->sendPaymentRejectionEmail($Order);
        }
        $userId = User::find($customer_id);
        $plan = Plan::where('id', $userId->active_plan_id)->first();
        if ($plan->is_free_plan == 1) {
            $hasPreviousFreePlan = User::where('id', $customer_id)
                ->where('free_plan', 1)
                ->exists();
            
            if ($hasPreviousFreePlan) {
                return $this->error('You have already used a free plan previously. Please choose a paid plan.', 422);
            }
        
            $user = User::find($customer_id);
            
            if ($user->free_plan == 0) {
                $user->free_plan = 1;
                $user->save();
            }
        }
        return $this->message('status update successfully');
    }

    /**
     * Send email notification to customer about pending bank transfer
     * 
     * @param Order $order
     * @return void
     */
    protected function sendBankTransferPendingEmail($order)
    {
        try {
            $user = \App\Models\User::find($order->user_id);
            if (!$user) {
                Log::error('User not found for order: ' . $order->order_id);
                return;
            }
            
            $plan = Plan::find($order->plan_id);
            if (!$plan) {
                Log::error('Plan not found for order: ' . $order->order_id);
                return;
            }
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Your Bank Transfer Request is Pending - ' . env('APP_NAME'),
                'msg' => view('Emails.bank-transfer-pending', [
                    'subject' => 'Your Bank Transfer Request is Pending',
                    'name' => $user->name ?? $order->name,
                    'plan_name' => $order->plan_name ?? $plan->name,
                    'order_id' => $order->order_id,
                    'price' => $order->price,
                    'currency' => $order->price_curency ?? 'USD',
                    'interval' => $order->interval
                ])->render()
            ];
            
            $emailSent = $this->SendInstantEmail($emailParams);
            if (!$emailSent) {
                Log::error('Failed to send bank transfer pending email for order: ' . $order->order_id);
            } else {
                Log::info('Bank transfer pending email sent for order: ' . $order->order_id);
            }
            
        } catch (\Exception $e) {
            Log::error('Error sending bank transfer pending email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }
    
    /**
     * Send notification to admin about new payment/order
     *
     * @param Order $order
     * @return void
     */
    protected function sendAdminPaymentNotification($order)
    {
        try {
            // Get admin email from settings
            $admin = User::where('user_type', config('constants.user_types.admin'))->first();
            $adminEmail = DB::table('adminsettings')
                ->where('customer_id', $admin->id)
                ->where('key', 'company_company_sales_email')
                ->value('value');
                
            if (!$adminEmail) {
                Log::warning('Admin sales email not found in settings');
                return;
            }
            
            $user = \App\Models\User::find($order->user_id);
            $userName = $user ? $user->name : ($order->name ?? 'Customer');
            $userEmail = $user ? $user->email : ($order->email ?? 'Unknown');
            
            $plan = Plan::find($order->plan_id);
            $planName = $plan ? $plan->name : ($order->plan_name ?? 'Unknown Plan');
            
            // Get payment status from order or use the isPending parameter as fallback
            $paymentStatus = isset($order->payment_status) ? (int) $order->payment_status : config('constants.payment_statuses.completed');
            
            // Define status-specific content based on payment status
            $statusConfig = $this->getPaymentStatusConfig($paymentStatus);
            
            // Get a descriptive payment type name
            $paymentType = $this->getPaymentTypeName($order->payment_type ?? config('constants.payment_types.bank'), $paymentStatus);
            
            $subject = $statusConfig['subject'];
                
            $emailParams = [
                'to' => $adminEmail,
                'subject' => $subject . ' - WMS',
                'msg' => view('Emails.admin-payment-notification', [
                    'subject' => $subject,
                    'order_id' => $order->order_id,
                    'customer_name' => $userName,
                    'customer_email' => $userEmail,
                    'plan_name' => $planName,
                    'price' => $order->price,
                    'currency' => $order->price_curency ?? 'USD',
                    'payment_type' => $paymentType,
                    'is_recurring' => $order->is_recurring ? 'Yes' : 'No',
                    'interval' => $order->interval ?? '1',
                    'date' => now()->format('Y-m-d H:i:s'),
                    // Pass the payment status and configuration to the template
                    'payment_status' => $paymentStatus,
                    'status_config' => $statusConfig
                ])->render()
            ];
            
            $emailSent = $this->SendInstantEmail($emailParams);
            if (!$emailSent) {
                Log::error('Failed to send admin payment notification for order: ' . $order->order_id);
            } else {
                Log::info('Admin payment notification sent for order: ' . $order->order_id);
            }
            
        } catch (\Exception $e) {
            Log::error('Error sending admin payment notification: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }
    
    /**
     * Get configuration for different payment statuses
     *
     * @param int $paymentStatus
     * @return array
     */
  

    /**
     * Send email notification to customer about rejected payment
     * 
     * @param Order $order
     * @return void
     */
    protected function sendPaymentRejectionEmail($order)
    {
        try {
            $user = \App\Models\User::find($order->user_id);
            if (!$user) {
                Log::error('User not found for order: ' . $order->order_id);
                return;
            }
            
            $plan = Plan::find($order->plan_id);
            if (!$plan) {
                Log::error('Plan not found for order: ' . $order->order_id);
                return;
            }
            
            // Get the status configuration for rejected payments
            $statusConfig = $this->getPaymentStatusConfig(config('constants.payment_statuses.rejected'));
            
            $emailParams = [
                'to' => $user->email,
                'subject' => 'Your Payment Request Has Been Rejected - ' . env('APP_NAME'),
                'msg' => view('Emails.payment-rejected', [
                    'subject' => 'Your Payment Request Has Been Rejected',
                    'name' => $user->name ?? $order->name,
                    'plan_name' => $order->plan_name ?? $plan->name,
                    'order_id' => $order->order_id,
                    'price' => $order->price,
                    'currency' => $order->price_curency ?? 'USD',
                    'interval' => $order->interval,
                    'payment_type' => $this->getPaymentTypeName($order->payment_type, $order->payment_status),
                    'status_config' => $statusConfig
                ])->render()
            ];
            
            $emailSent = $this->SendInstantEmail($emailParams);
            if (!$emailSent) {
                Log::error('Failed to send payment rejection email for order: ' . $order->order_id);
            } else {
                Log::info('Payment rejection email sent for order: ' . $order->order_id);
            }
            
        } catch (\Exception $e) {
            Log::error('Error sending payment rejection email: ' . $e->getMessage());
            Log::error($e->getTraceAsString());
        }
    }
    
    /**
     * Send email notification to customer about failed payment
     * 
     * @param Order $order
     * @return void
     */
  
}
