<?php

namespace App\Http\Controllers;

use PDF;
use Illuminate\Support\Facades\DB;
use App\Models\WhsReport;
use App\Models\WhsReportItem;
use App\Models\GeneratedPdfReport;

class WhsReportController extends Controller
{

    public function whsReportDownloadPdfRegenerate($id)
    {
        if (!$id) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Download ID is required',
                'report_id' => 0,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error('Report ID is required.', 422);
        }
        try {
            // Fetch WHS report and add employee/customer checks
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Fetch all related data based on customer/employee
            $customerId = auth()->user()->customer_id ?? auth()->id();
            $workspaceId = auth()->user()->workspace_id ?? auth()->user()->current_workspace_id;
            $whs_data = DB::table('whs_reports')
                ->leftJoin('emp_personal_details as authorised_by', 'whs_reports.authorised_by', '=', 'authorised_by.id')
                ->leftJoin('projects as projects', 'whs_reports.project_id', '=', 'projects.id')
                ->leftJoin('swms as swms', 'whs_reports.swms_id', '=', 'swms.id')
                ->where('whs_reports.id', $id)
                ->where('whs_reports.customer_id', $customerId)
                ->where('whs_reports.workspace_id', $workspaceId)
                ->select(
                    'whs_reports.*',
                    'projects.title as project_title',
                    'swms.title as swms_title',
                    'swms.swms_number as swms_number',
                    'swms.revision_number as swms_revision_number',
                    'swms.version_number as version_number',
                    'authorised_by.first_name as authorised_by_first_name',
                    'authorised_by.last_name as authorised_by_or_forman_last_name'
                )->first();
            if (!$whs_data) {
                return $this->error('WHS report not found or not accessible.', 404);
            }
            $positions_responsible = DB::table('whs_responsible')
                ->join('roles', 'whs_responsible.role_id', '=', 'roles.id')
                ->where('whs_responsible.whs_report_id', $id)
                ->select('roles.*')
                ->get();
            $personal_attending = DB::table('whs_additional_persons')
                ->join('emp_personal_details', 'whs_additional_persons.employee_id', '=', 'emp_personal_details.id')
                ->where('whs_additional_persons.whs_report_id', $id)
                ->select('emp_personal_details.*')
                ->get();
            $whs_conducted_by = DB::table('whs_conducted_by')
                ->join('emp_personal_details', 'whs_conducted_by.employee_id', '=', 'emp_personal_details.id')
                ->where('whs_conducted_by.whs_report_id', $id)
                ->select('emp_personal_details.*')
                ->get();
            $whs_item = DB::table('whs_items')
                ->where('customer_id', $customerId)
                ->where('status', 1)
                ->get();
            $whs_hazards = DB::table('whs_report_hazards')
                ->join('roles', 'whs_report_hazards.role_id', '=', 'roles.id')
                ->where('whs_report_hazards.whs_report_id', $id)
                ->select('whs_report_hazards.*', 'roles.title as role_title')
                ->get();
            $report_images = DB::table('whs_images')
                ->where('whs_report_id', $id)
                ->get();
            $whsReportItems = WhsReportItem::where('whs_report_id', $id)->get()->keyBy('whs_items_id')->map(function ($item) {
                return $item->values;
            });
            // Render the PDF view
            $viewPath = 'WHSreport.NewWHSreport.PdfReport.whsReport';
            $html = view($viewPath, compact(
                'whs_item',
                'whs_data',
                'positions_responsible',
                'personal_attending',
                'whs_conducted_by',
                'whsReportItems',
                'whs_hazards',
                'report_images'
            ))->render();
            // Generate the filename
            $report_whs_number = $whs_data->number ?? 'NO_WHS';
            $report_title = $whs_data->title ?? 'No_Named';
            $report_whs_revision = $whs_data->revision_number ?? 'No_Revision';
            $filename = "{$report_title}_N{$report_whs_number}_R{$report_whs_revision}.pdf";
            // Ensure the directory exists
            $directory = public_path('WHSReportPdf');
            if (!file_exists($directory)) {
                mkdir($directory, 0777, true);
            }
            $path = $directory . '/' . $filename;
            // Delete existing file if already exists
            if (file_exists($path)) {
                unlink($path);
            }
            // Generate and save the PDF
            $pdf = PDF::loadHTML($html)
                ->setPaper('A4', 'portrait');
            $pdf->save($path);
            // Save the generated PDF record
            GeneratedPdfReport::updateOrCreate(
                [
                    'report_id' => $id,
                    'report_type' => 'whs_report',
                ],
                ['path' => url("WHSReportPdf/{$filename}")]
            );
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS PDF generated successfully',
                'report_id' => $id,
                'report_type' => 'whs_report',
            ]);
            return $this->success(['download_url' => url("WHSReportPdf/{$filename}")], 'PDF generated successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS PDF generation error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while generating the PDF.' . $e->getMessage(), 500);
        }
    }


    // PDF
    public function whsReportDownloadPdf($id)
    {
        if (!$id) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Download ID is required',
                'report_id' => 0,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error('Report ID is required.', 422);
        }
        try {
            // Fetch WHS report and add employee/customer checks
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Fetch related data
            $customerId = auth()->user()->customer_id ?? auth()->id();
            $workspaceId = auth()->user()->workspace_id ?? auth()->user()->current_workspace_id;
            $whs_data = DB::table('whs_reports')
                ->leftJoin('emp_personal_details as authorised_by', 'whs_reports.authorised_by', '=', 'authorised_by.id')
                ->leftJoin('projects as projects', 'whs_reports.project_id', '=', 'projects.id')
                ->leftJoin('swms as swms', 'whs_reports.swms_id', '=', 'swms.id')
                ->where('whs_reports.id', $id)
                ->where('whs_reports.customer_id', $customerId)
                ->where('whs_reports.workspace_id', $workspaceId)
                ->select(
                    'whs_reports.*',
                    'projects.title as project_title',
                    'swms.title as swms_title',
                    'swms.swms_number as swms_number',
                    'swms.revision_number as swms_revision_number',
                    'swms.version_number as version_number',
                    'authorised_by.first_name as authorised_by_first_name',
                    'authorised_by.last_name as authorised_by_or_forman_last_name'
                )->first();
            $positions_responsible = DB::table('whs_responsible')
                ->join('roles', 'whs_responsible.role_id', '=', 'roles.id')
                ->where('whs_responsible.whs_report_id', $id)
                ->select('roles.*')
                ->get();
            $personal_attending = DB::table('whs_additional_persons')
                ->join('emp_personal_details', 'whs_additional_persons.employee_id', '=', 'emp_personal_details.id')
                ->where('whs_additional_persons.whs_report_id', $id)
                ->select('emp_personal_details.*')
                ->get();
            $whs_conducted_by = DB::table('whs_conducted_by')
                ->join('emp_personal_details', 'whs_conducted_by.employee_id', '=', 'emp_personal_details.id')
                ->where('whs_conducted_by.whs_report_id', $id)
                ->select('emp_personal_details.*')
                ->get();
            $whs_hazards = DB::table('whs_report_hazards')
                ->join('roles', 'whs_report_hazards.role_id', '=', 'roles.id')
                ->where('whs_report_hazards.whs_report_id', $id)
                ->select('whs_report_hazards.*', 'roles.title as role_title')
                ->get();
            $whs_item = DB::table('whs_items')
                ->where('customer_id', $customerId)
                ->where('status', 1)
                ->get();
            $whsReportItems = WhsReportItem::where('whs_report_id', $id)->get()->keyBy('whs_items_id')->map(function ($item) {
                return $item->values;
            });
            $report_images = DB::table('whs_images')->where('whs_report_id', $id)->get();
            // Render the PDF view
            $viewPath = 'WHSreport.NewWHSreport.PdfReport.whsReport';
            $html = view($viewPath, compact(
                'whs_data',
                'positions_responsible',
                'personal_attending',
                'whs_conducted_by',
                'whs_item',
                'whsReportItems',
                'whs_hazards',
                'report_images'
            ))->render();
            // Generate the filename
            $report_whs_number = $whs_data->number ?? 'NO_WHS';
            $report_title = $whs_data->title ?? 'No_Named';
            $report_whs_revision = $whs_data->revision_number ?? 'No_Revision';
            $filename = "{$report_title}_N{$report_whs_number}_R{$report_whs_revision}.pdf";
            // Ensure the directory exists
            $directory = public_path('WHSReportPdf');
            if (!file_exists($directory)) {
                mkdir($directory, 0777, true);
            }
            $path = $directory . '/' . $filename;
            // Check if file already exists
            if (file_exists($path)) {
                return $this->success(['download_url' => url("WHSReportPdf/{$filename}")], 'PDF already exists.');
            }
            // Generate and save the PDF
            $pdf = PDF::loadHTML($html)->setPaper('A4', 'portrait');
            $pdf->save($path);
            // Save the generated PDF record
            GeneratedPdfReport::updateOrCreate(
                [
                    'report_id' => $id,
                    'report_type' => 'whs_report',
                ],
                ['path' => url("WHSReportPdf/{$filename}")]
            );
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS PDF generated successfully',
                'report_id' => $id,
                'report_type' => 'whs_report',
            ]);
            return $this->success(['download_url' => url("WHSReportPdf/{$filename}")], 'PDF generated successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS PDF generation error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while generating the PDF.', 500);
        }
    }


    public function whsReportPublishReport($id)
    {
        if (!$id) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Publish ID is required',
                'report_id' => 0,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error('Report ID is required.', 422);
        }
        try {
            // Fetch WHS report and add employee/customer checks
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Report Not Found',
                    'report_id' => $id,
                    'report_type' => 'whs_report',
                    'error_type' => 'Not found error',
                ]);
                return $this->error('Report not found. Please try again.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Generate PDF if necessary
            $this->whsReportDownloadPdf($id);
            $whsReport->is_published = $whsReport->is_published == '1' ? '0' : '1';
            $whsReport->save();
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Report status updated successfully',
                'report_id' => $id,
                'report_type' => 'whs_report',
            ]);
            return $this->success($whsReport, 'Report status updated successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Error while publishing report: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while publishing the report.', 500);
        }
    }

    public function whsReportViewSignature($id)
    {
        if (!$id) {
            return $this->error('Report ID is required.', 422);
        }
        try {
            // Fetch WHS report and add employee/customer checks
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Fetch WHS data
            $whs_data = DB::table('whs_reports')
                ->leftJoin('emp_personal_details as authorised_by', 'whs_reports.authorised_by', '=', 'authorised_by.emp_id')
                ->where('whs_reports.id', $id)
                ->select(
                    'whs_reports.*',
                    'authorised_by.first_name as authorised_by_first_name',
                    'authorised_by.last_name as authorised_by_or_forman_last_name'
                )->first();
            // Fetch WHS signature data
            $whs_signature_data = DB::table('whs_signatures')
                ->leftJoin('emp_personal_details as authorised_by', 'whs_signatures.employee_id', '=', 'authorised_by.emp_id')
                ->where('whs_signatures.whs_id', $id)
                ->select(
                    'whs_signatures.*',
                    'authorised_by.first_name as first_name',
                    'authorised_by.last_name as last_name'
                )->get();
            // Prepare data for the response
            $data = [
                'whs_data' => $whs_data,
                'whs_signature_data' => $whs_signature_data,
            ];
            return $this->success($data, 'WHS Report Signature data retrieved successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Signature View Error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving the WHS report signature data.', 500);
        }
    }
}
