<?php

namespace App\Http\Controllers;

use Exception;
use App\Models\Role;
use App\Models\Swms;
use App\Models\Project;
use App\Models\WhsItem;
use App\Models\WhsImages;
use App\Models\WhsReport;
use Illuminate\Http\Request;
use App\Models\WhsReportItem;
use App\Models\WhsConductedBy;
use App\Models\WhsResponsible;
use App\Models\WhsReportHazards;
use App\Models\EmpCompanyDetails;
use App\Models\GeneratedPdfReport;
use Illuminate\Support\Facades\DB;
use App\Models\WhsAdditionalPerson;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\Validator;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Str; // Import Str facade




class WhsStepsController extends Controller
{
    public function whsStep1Store(Request $request)
    {
        $isUpdate = $request->whs_report_id;
        // Validation rules
        $rules = [
            'document_title' => 'required|string|max:255',
            'document_number' => 'nullable|string|max:255',
            'authorised_by' => 'nullable|integer|max:255',
            'revision_number' => 'nullable|string|max:255',
            'revision_date' => 'nullable|string|max:255',
            'role_id' => 'nullable|string|max:20',
            'priority' => 'nullable|string|max:255',
            'site_id' => 'required',
        ];
        $rules['issue_date'] = $isUpdate != 0 ? 'nullable|date' : 'nullable|date|after_or_equal:today';
        // Validation
        $validator = Validator::make($request->all(), $rules);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 1 error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $message = $isUpdate ? 'Updated Successfully' : 'Saved Successfully';
        $userTable = $this->getUserTable(); // Get the user type (customer or emp)
        if ($userTable == "customer") {
            $customer_id = auth()->id();
            $workspace_id = auth()->user()->current_workspace_id;
        } elseif ($userTable == "emp") {
            $customer_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
        }
        try {
            $data = [
                'title' => $validator->validated()['document_title'],
                'number' => $validator->validated()['document_number'],
                'authorised_by' => $validator->validated()['authorised_by'],
                'revision_number' => $validator->validated()['revision_number'],
                'issue_date' => $validator->validated()['issue_date'],
                'revision_date' => $validator->validated()['revision_date'],
                'priority' => $validator->validated()['priority'],
                'site_id' => $validator->validated()['site_id'],
                'created_by' => auth()->user()->id,
                'customer_id' => $customer_id,
                'workspace_id' => $workspace_id,
            ];
            $already_report = WhsReport::find($request->whs_report_id);
            $already_number_revision = WhsReport::where('revision_number', $validator->validated()['revision_number'])
                ->where('number', $validator->validated()['document_number'])
                ->where('del', 0)
                ->count();
            if ($already_number_revision > 1) {
                return $this->errorWithData($data, 'Document Number and Revision Number Already Exist', 409);
            }
            if ($isUpdate == 0 && $already_number_revision > 0) {
                return $this->errorWithData($data, 'Document Number and Revision Number Already Exist', 422);
            }
            if ($already_report && $validator->validated()['revision_number'] != $already_report->revision_number) {
                $data = [
                    'title' => $already_report->title,
                    'number' => $already_report->number,
                    'authorised_by' => $already_report->authorised_by,
                    'revision_number' => $already_report->revision_number,
                    'issue_date' => $already_report->issue_date,
                    'revision_date' => $already_report->revision_date,
                    'priority' => $already_report->priority,
                    'site_id' => $already_report->site_id,
                    'created_by' => auth()->user()->id,
                    'customer_id' => $already_report->customer_id,
                    'workspace_id' => $already_report->workspace_id,
                ];
            }
            $whs = WhsReport::updateOrCreate(
                ['id' => $request->whs_report_id],
                $data
            );
            $whs_id = $whs->id;
            if ($already_report && $validator->validated()['revision_number'] != $already_report->revision_number) {
                $new_whs_report = $whs->replicate();
                $new_whs_report->title = $validator->validated()['document_title'];
                $new_whs_report->number = $validator->validated()['document_number'];
                $new_whs_report->authorised_by = $validator->validated()['authorised_by'];
                $new_whs_report->revision_number = $validator->validated()['revision_number'];
                $new_whs_report->issue_date = $validator->validated()['issue_date'];
                $new_whs_report->revision_date = $validator->validated()['revision_date'];
                $new_whs_report->priority = $validator->validated()['priority'];
                $new_whs_report->site_id = $request->site_id;
                $new_whs_report->created_by = auth()->user()->id;
                $new_whs_report->customer_id = $customer_id;
                $new_whs_report->workspace_id = $workspace_id;
                $new_whs_report->save();
                $this->whsReportDuplicate($new_whs_report->id, $whs->id);
                $whs_id = $new_whs_report->id;
            }
            $data['whs_report_id'] = $whs_id;
            if ($request->role_id) {
                WhsResponsible::where('whs_report_id', $whs_id)->delete();
                $role_ids = explode(',', $request->role_id);
                foreach ($role_ids as $id) {
                    WhsResponsible::firstOrCreate([
                        'whs_report_id' => $whs_id,
                        'role_id' => $id,
                    ]);
                }
            }
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 1 ' . $message,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ];
            storeReportsLogs($log);
            return $this->success($data, $message);
        } catch (\Exception $e) {
            // Handle exceptions and return an error response
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Step 1 error:' . $shortMessage,
                'report_id' =>  $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred while saving  data:');
        }
    }

    public function whsStep1Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 1 edit validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Get user type and find the WHS report
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->with('siteData')->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data = $whsReport;
            if ($whsReport) {
                $selectedRoleIds = WhsResponsible::where('whs_report_id', $request->whs_report_id)->pluck('role_id');
                if ($selectedRoleIds) {
                    $data['Whs_responsible'] = Role::whereIn('id', $selectedRoleIds)->pluck('id');
                }
            }
            // Fetch roles and authorized personnel
            $data['role'] = Role::where('del', 0)->get();
            $data['authorised_by_list'] = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                ])
                ->select('id', 'access_role', 'tier_id')
                ->get();
            // Fetch sites
            $data['sites'] = DB::table('sites')
                ->where('del', '0')
                ->where(function ($query) use ($whsReport) {
                    $query->where('active', '1')
                        ->orWhere(function ($subQuery) use ($whsReport) {
                            if ($whsReport && isset($whsReport->site_id)) {
                                $subQuery->where('id', $whsReport->site_id);
                            }
                        });
                })
                ->get(['id', 'title']);
            $data['whs_report_id'] = $request->whs_report_id;
            return $this->success($data, 'Record retrieved successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Step 1 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving data: ' . $e->getMessage());
        }
    }

    public function whsStep2Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
            'project_id' => 'nullable|integer',
            'date' => 'nullable|date',
            'start_time' => 'nullable|string',
            'end_time' => 'nullable|string',
            'swms_id' => 'nullable|integer',
            'location' => 'nullable|string|max:255',
            'activities' => 'nullable|string',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 2 validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        // Check customer and employee access
        $userTable = $this->getUserTable();
        $whsReport = WhsReport::find($request->whs_report_id);
        if (!$whsReport) {
            return $this->message('WHS report not found.', 404);
        }
        if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        $start_time = $request->start_time ? date('H:i:s', strtotime($request->start_time)) : null;
        $end_time = $request->end_time ? date('H:i:s', strtotime($request->end_time)) : null;
        // Prepare the data for storage
        $data = [
            'id' => $request->whs_report_id,
            'project_id' => $request->project_id,
            'date' => $request->date,
            'start_time' => $start_time,
            'end_time' => $end_time,
            'swms_id' => $request->swms_id,
            'location' => $request->location,
            'activities' => $request->activities,
        ];
        try {
            // Update or create the WHS meta data
            $whs = WhsReport::updateOrCreate(
                ['id' => $request->whs_report_id],
                $data
            );
            if ($request->additional_attending_inspections_id) {
                WhsAdditionalPerson::where('whs_report_id', $whs->id)->delete();
                $additional_attending_inspections_ids = explode(",", $request->additional_attending_inspections_id);

                foreach ($additional_attending_inspections_ids as $id) {
                    WhsAdditionalPerson::create([
                        'whs_report_id' => $whs->id,
                        'employee_id' => $id,
                    ]);
                }
            }
            if ($request->conducted_by) {
                WhsConductedBy::where('whs_report_id', $whs->id)->delete();
                $conducted_bys = explode(",", $request->conducted_by);

                foreach ($conducted_bys as $id) {
                    WhsConductedBy::create([
                        'whs_report_id' => $whs->id,
                        'employee_id' => $id,
                    ]);
                }
            }
            $data['whs_report_id'] = $request->whs_report_id;
            $message = $request->whs_report_id ? 'Updated Successfully' : 'Saved Successfully';
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 2 ' . $message,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report'
            ];
            storeReportsLogs($log);
            return $this->success($data, $message);
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 2 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred while saving the data.');
        }
    }


    public function whsStep2Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 2 edit validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Get user type and fetch the WHS report
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data = $whsReport;
            // Fetch additional persons
            $WhsAdditionalPersonIds = WhsAdditionalPerson::where('whs_report_id', $request->whs_report_id)->pluck('employee_id');
            $data['Whs_additional_person'] = EmpCompanyDetails::whereIn('id', $WhsAdditionalPersonIds)->pluck('id');
            // Fetch conducted by
            $WhsConductedByIds = WhsConductedBy::where('whs_report_id', $request->whs_report_id)->pluck('employee_id');
            $data['Whs_conducted_by'] = EmpCompanyDetails::whereIn('id', $WhsConductedByIds)->pluck('id');
            // Fetch authorized by list
            $data['authorised_by_list'] = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                ])
                ->select('id', 'access_role', 'tier_id')
                ->get();
            // Fetch projects and SWMS
            $data['project'] = Project::where('is_deleted', 0)->get();
            $data['swms'] = Swms::all();
            $data['whs_report_id'] = $request->whs_report_id;
            return $this->success($data, 'Record retrieved successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 2 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving data: ' . $e->getMessage(), 500);
        }
    }

    public function whsStep3Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
            'results' => 'required|array',
            'results.*.id' => 'required|integer',
            'results.*.response' => 'required|string|in:0,1,2', // Only allow valid ENUM values
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 store validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and add customer/employee access check
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Process the results array
            $results = $request->input('results');
            foreach ($results as $res) {
                WhsReportItem::updateOrCreate(
                    [
                        'whs_report_id' => $request->whs_report_id, // Matching condition (unique fields)
                        'whs_items_id' => $res['id'],              // Matching condition (unique fields)
                    ],
                    [
                        'values' => $res['response'],              // Ensure valid ENUM value
                    ]
                );
            }
            $message = $request->whs_report_id ? 'Updated Successfully' : 'Saved Successfully';
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3: ' . $message,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ]);
            return $this->success([
                'whs_report_id' => $request->whs_report_id,
                'results' => $results
            ], $message);
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while processing the data.', 500);
        }
    }

    public function whsStep3Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 edit validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Get WHS report and check customer/employee access
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Fetch WHS report items and map them properly
            $whsReportItems = WhsReportItem::where('whs_report_id', $request->whs_report_id)
                ->get()
                ->keyBy('whs_items_id')
                ->map(function ($item) {
                    return $item->values; // Only return 'values' field
                });
            // Fetch WHS items that are active (status = 1)
            $data = [
                'whs_items' => WhsItem::where('status', '1')->get(),
                'whs_report_items' => $whsReportItems,
            ];
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 data retrieved successfully.',
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ]);
            return $this->success($data, 'Record retrieved successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while fetching the data.', 500);
        }
    }


    public function whsStep4Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 4 store validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and add customer/employee access check
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Get Input Data
            $whs_report_id = $request->whs_report_id;
            $column_role_ids = $request->input('column_role_id', []);
            $levels = $request->input('column_level', []);
            $hazards = $request->input('column_hazard', []);
            $actions_required = $request->input('column_action_required', []);
            $levels_of_action = $request->input('column_level_of_action', []);
            $dates_completed = $request->input('column_date_completed', []);
            $whsReportIds = $request->input('column_whs_hazard_report_id', []);
            foreach ($column_role_ids as $index => $responsiblePersonId) {
                $level = $levels[$index] ?? null;
                $hazard = $hazards[$index] ?? null;
                $action_required = $actions_required[$index] ?? null;
                $level_of_action = $levels_of_action[$index] ?? null;
                $date_completed = $dates_completed[$index] ?? null;
                $whsReportId = $whsReportIds[$index] ?? null;
                if ($whsReportId) {
                    // Update existing record
                    WhsReportHazards::updateOrCreate(
                        ['id' => $whsReportId, 'whs_report_id' => $whs_report_id],
                        [
                            'role_id' => $responsiblePersonId,
                            'level' => $level,
                            'hazard' => $hazard,
                            'action_required' => $action_required,
                            'level_of_action' => $level_of_action,
                            'date_completed' => $date_completed,
                        ]
                    );
                } else {
                    // Create a new record
                    WhsReportHazards::create([
                        'whs_report_id' => $whs_report_id,
                        'role_id' => $responsiblePersonId,
                        'level' => $level,
                        'hazard' => $hazard,
                        'action_required' => $action_required,
                        'level_of_action' => $level_of_action,
                        'date_completed' => $date_completed,
                    ]);
                }
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 4: Saved Successfully',
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ]);
            return $this->success($whs_report_id, 'Saved Successfully');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 4 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while saving WHS data.', 500);
        }
    }

    public function whsStep4Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 4 edit validation error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and add customer/employee access check
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->message('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Fetch Data
            $data = [
                'role' => Role::where('del', 0)->get(),
                'WhsReportHazards' => WhsReportHazards::where('whs_report_id', $request->whs_report_id)->get(),
                'responsible_person_list' => EmpCompanyDetails::where('compeleted', '1')
                    ->where('approved', '1')
                    ->where('status', '1')
                    ->where('del', '0')
                    ->with([
                        'empPersonalDetails' => fn($query) => $query->select('emp_id', 'first_name', 'middle_name', 'last_name'),
                        'accessRole' => fn($query) => $query->select('id', 'title', 'code'),
                        'empTier' => fn($query) => $query->select('id', 'title'),
                    ])
                    ->select('id', 'access_role', 'tier_id')
                    ->get(),
                'whs_report_id' => $request->whs_report_id,
            ];
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Record retrieved successfully',
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ]);
            return $this->success($data, 'Record retrieved successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 4 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving WHS data.', 500);
        }
    }

    public function whsPlanSettingRoleList()
    {
        try {
            $role = Role::where('del', 0)->get();
            $data['role'] = $role;
            if ($role->isEmpty()) {
                return $this->message('No Role found please create.', 404);
            }
            return $this->success($data, 'Records get successfully');
        } catch (\Exception $e) {
            return $this->error($e->getMessage());
        }
    }
    public function whsStep3HazardsDelete($id)
    {
        $whsReportHazards = WhsReportHazards::find($id);
        if (!$whsReportHazards) {
            return $this->error('Hazard not found.', 404);
        }
        $whsReport = WhsReport::find($whsReportHazards->whs_report_id);
        if (!$whsReport) {
            return $this->error('Associated WHS report not found.', 404);
        }
        $userTable = $this->getUserTable();
        // Customer and Employee Access Check
        if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->error('You do not have access to delete this hazard.', 403);
        }
        if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
            return $this->error('You do not have access to delete this hazard.', 403);
        }
        $whsReportHazards->delete();
        return $this->message('Hazard deleted successfully.');
    }

    public function whsStep5Store(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
            'general_notes' => 'nullable|string|max:255',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 Store Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and check authorization
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Update or create WHS record
            $whs = WhsReport::updateOrCreate(
                ['id' => $request->whs_report_id],
                ['general_notes' => $request->general_notes]
            );
            // Log success
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 Saved Successfully',
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report'
            ]);
            return $this->success($whs, 'Saved Successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while saving WHS data.', 500);
        }
    }

    public function whsStep5Edit(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 Edit Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and check authorization
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            $data = $whsReport;
            $data['whs_report_id'] = $request->whs_report_id;
            // Log success
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 Record Retrieved Successfully',
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report'
            ]);
            return $this->success($data, 'Record retrieved successfully');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving WHS data.', 500);
        }
    }


    public function whsReportListData(Request $request)
    {
        try {
            $auth_id = auth()->user()->id;
            $userTable = $this->getUserTable();
            $query = WhsReport::query()
                ->orderBy('id', 'DESC')
                ->where('del', '0')
                ->with('authorizedBy');
            if ($userTable === 'customer') {
                $query->where('customer_id', $auth_id)
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            } elseif ($userTable === 'emp') {
                $query->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            } else {
                return $this->error('You do not have permission to access these reports.', 403);
            }
            // Apply filters if provided
            $filtersApplied = null;
            if ($request->filled('filter')) {
                $filtersApplied = json_decode($request->filter, true);
                $query = $this->filter($filtersApplied, $query);
            }
            $start = $request->input('from', 0);
            $query->offset($start)->limit(20);
            $query_result = $query->get();
            return $this->WithFilter($query_result, $filtersApplied ?? null, 'WHS Reports List Retrieved Successfully');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report List Error: ' . $shortMessage,
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while retrieving WHS reports.', 500);
        }
    }



    public function filter($filters, $query)
    {
        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'authorised_by':
                        $filterValue  = explode(" ", $filterValue);
                        if (count($filterValue) == 1) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
                        if (count($filterValue) == 2) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
                        if (count($filterValue) == 3) {

                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'document_number':
                        $query->where('number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'revision_number':
                        $query->where('revision_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'issue_date':
                        $query->where('issue_date', 'like', '%' . $filterValue . '%');
                        break;
                    case 'process':
                        $query->where('status', 'like', '%' . $filterValue . '%');
                        break;
                    case 'published':
                        $query->where('is_published', 'like', '%' . $filterValue . '%');
                        break;
                }
            }
        }
        return $query;
    }


    public function destroy($id)
    {
        try {
            $auth_id = auth()->user()->id;
            $userTable = $this->getUserTable();
            // Find the WHS report
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                return $this->error('WHS Report not found.', 404);
            }
            if ($whsReport->del == 1) {
                return $this->error('You cannot delete this report because it has been deleted.', 403);
            }
            // Customer & Employee Access Check
            if ($userTable === 'customer') {
                if ($whsReport->customer_id !== $auth_id || $whsReport->workspace_id !== auth()->user()->current_workspace_id) {
                    return $this->error('You do not have permission to delete this report.', 403);
                }
            } elseif ($userTable === 'emp') {
                if ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id) {
                    return $this->error('You do not have permission to delete this report.', 403);
                }
            }
            // Log the deletion
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Deleted',
                'report_id' => $id,
                'report_type' => 'whs_report'
            ]);
            // Soft delete the WHS report
            $whsReport->del = 1;
            $whsReport->save();
            return $this->message('WHS Report Deleted Successfully');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Delete Error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('An error occurred while deleting the WHS report.', 500);
        }
    }

    public function whsReportDuplicate($new_report_id, $old_report_id)
    {
        if (!$new_report_id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Step 1 duplicate not found.',
                'report_id' =>  $old_report_id,
                'report_type' => 'whs_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'New Revision Report id not found. Please try again',
                'new_report_id' => $new_report_id,
                'old_report_id' => $old_report_id,
            ], 433);
        }
        try {
            // Retrieve existing records
            $Whs_responsible_step1 = WhsResponsible::where('whs_report_id', $old_report_id)->get();
            $Whs_additional_person_step2 = WhsAdditionalPerson::where('whs_report_id', $old_report_id)->get();
            $Whs_conducted_by_step2 = WhsConductedBy::where('whs_report_id', $old_report_id)->get();
            $whs_report_item_step3 = WhsReportItem::where('whs_report_id', $old_report_id)->get();
            $Whs_report_hazards_step4 = WhsReportHazards::where('whs_report_id', $old_report_id)->get();
            // Duplicate records with new swms_id
            $this->duplicateRecords($Whs_responsible_step1, $new_report_id);
            $this->duplicateRecords($Whs_additional_person_step2, $new_report_id);
            $this->duplicateRecords($Whs_conducted_by_step2, $new_report_id);
            $this->duplicateRecords($whs_report_item_step3, $new_report_id);
            $this->duplicateRecords($Whs_report_hazards_step4, $new_report_id);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report ID' . $old_report_id . ' duplicated successfully',
                'report_id' =>  $new_report_id,
                'report_type' => 'whs_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Report duplicated successfully',
                'new_report_id' => $new_report_id,
                'old_report_id' => $old_report_id,
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report duplicated error:' . $shortMessage,
                'report_id' =>  $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }

    private function duplicateRecords($records, $new_report_id)
    {
        foreach ($records as $record) {
            $recordClone = $record->replicate();
            $recordClone->whs_report_id = $new_report_id;
            $recordClone->save();
        }
    }

    // need to confirm this method use or not
    public function whsReportPdfFilesManager(Request $request)
    {
        // Get the current page
        $currentPage = Paginator::resolveCurrentPage();
        // Number of items per page
        $perPage = 7;
        // Fetch SWMS report files from the database
        $query = GeneratedPdfReport::where('report_type', 'whs_report');
        // Count total items
        $totalReports = $query->count();
        // Get the reports for the current page
        $reports = $query->skip(($currentPage - 1) * $perPage)->take($perPage)->get();
        $paginator = new LengthAwarePaginator(
            $reports,
            $totalReports,
            $perPage,
            $currentPage,
            ['path' => Paginator::resolveCurrentPath()]
        );
        return $this->success($paginator);
    }


    // need to confirm this method logic also
    public function fileManagerDownloadWhsPdfFile($id, $filename)
    {
        // Validate request parameters
        if (!$id || !$filename) {
            return $this->error('Report ID and filename are required.', 422);
        }
        try {
            // Ensure the filename has a .pdf extension
            if (!Str::endsWith(strtolower($filename), '.pdf')) {
                $filename .= '.pdf';
            }
            // Check if the report exists and add customer/employee validation
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($id);
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Set file path
            $path = public_path("WHSReportPdf/{$filename}");
            // Check if file exists
            if (!file_exists($path)) {
                $cacheKey = 'whs_report_log_' . auth()->user()->id . '_' . $id;
                if (!cache()->has($cacheKey)) {
                    storeReportsLogs([
                        'employee_id' => auth()->user()->id,
                        'message' => 'WHS File not found',
                        'report_id' => $id,
                        'report_type' => 'whs_report'
                    ]);
                    cache()->put($cacheKey, true, now()->addSeconds(15));
                }
                return $this->error('WHS file not found.', 404);
            }
            // Log the download event if not already logged recently
            $cacheKey = 'whs_report_log_' . auth()->user()->id . '_' . $id;
            if (!cache()->has($cacheKey)) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'WHS File Successfully Downloaded',
                    'report_id' => $id,
                    'report_type' => 'whs_report'
                ]);
                cache()->put($cacheKey, true, now()->addSeconds(15));
            }
            // Generate a public URL for the file
            $downloadUrl = url("WHSReportPdf/{$filename}");
            return $this->success($downloadUrl, 'WHS file download URL generated successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS File download error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('An error occurred while processing the file download.');
        }
    }

    public function imageStore(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
            'image' => 'required|mimes:jpeg,png,jpg,gif,svg|max:10240',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Image Upload Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        // Fetch WHS report and add customer/employee access check
        $userTable = $this->getUserTable();
        $whsReport = WhsReport::find($request->whs_report_id);
        if (!$whsReport) {
            return $this->error('WHS report not found.', 404);
        }
        if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->error('You do not have access to this report.', 403);
        }
        if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
            return $this->error('You do not have access to this report.', 403);
        }
        try {
            // Check and upload the image
            if ($request->hasFile('image') && $request->file('image')->isValid()) {
                $imagePath = $this->handleFileImageUpload($request, 'WHSReportImages')['path'] ?? null;
                $image = new WhsImages();
                $image->image = $imagePath;
                $image->whs_report_id = $request->whs_report_id;
                $image->save();

                $images = WhsImages::where('whs_report_id', $request->whs_report_id)->get();
                return $this->success($images, 'Image uploaded successfully');
            }
            return $this->error('Invalid or no image uploaded.', 400);
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Image Upload Error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while uploading the image.', 500);
        }
    }


    public function getImages(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Image Fetch Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and add customer/employee access check
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::find($request->whs_report_id);
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            $images = WhsImages::where('whs_report_id', $request->whs_report_id)->get();

            return $this->success($images, 'Images retrieved successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Image Fetch Error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while fetching images.', 500);
        }
    }

    public function deleteImage($id)
    {
        try {
            // Find the image by ID
            $image = WhsImages::find($id);
            if (!$image) {
                return $this->error('Image not found.', 404);
            }
            // Fetch WHS report associated with the image
            $whsReport = WhsReport::find($image->whs_report_id);
            if (!$whsReport) {
                return $this->error('Associated WHS report not found.', 404);
            }
            // Customer and Employee Access Check
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this image.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this image.', 403);
            }
            // Delete the image file from storage
            $imagePath = public_path($image->image);
            if (file_exists($imagePath)) {
                unlink($imagePath); // Delete the file
            }
            // Delete the image record from the database
            $image->delete();
            // Log the deletion
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Image Deleted',
                'report_id' => $whsReport->id,
                'report_type' => 'whs_report',
            ]);
            // Return success response
            return $this->message('Image deleted successfully.');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Report Image Deletion Error: ' . $shortMessage,
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while deleting the image.', 500);
        }
    }


    public function whsStep6Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'whs_report_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 6 Store Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // Fetch WHS report and check access
            $userTable = $this->getUserTable();
            $whsReport = WhsReport::where('id', $request->whs_report_id)->first();
            if (!$whsReport) {
                return $this->error('WHS report not found.', 404);
            }
            if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->error('You do not have access to this report.', 403);
            }
            // Update or create the WHS report
            $data = ['status' => 1];
            $whs = WhsReport::updateOrCreate(['id' => $request->whs_report_id], $data);
            // Log the action
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 6: ' . ($request->whs_report_id ? 'Updated Successfully' : 'Saved Successfully'),
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
            ]);
            return $this->success($whs, $request->whs_report_id ? 'Updated Successfully' : 'Saved Successfully');
        } catch (\Throwable $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 6 Error: ' . $shortMessage,
                'report_id' => $request->whs_report_id,
                'report_type' => 'whs_report',
                'error_type' => 'Exception error',
            ]);
            return $this->error('An error occurred while saving data.', 500);
        }
    }
}
