<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Carbon;

class PasswordResetOtp extends Model
{
    use HasFactory;
    
    protected $fillable = [
        'email', 
        'otp', 
        'expires_at',
        'used',
        'token',
    ];
    
    protected $casts = [
        'expires_at' => 'datetime',
        'used' => 'boolean'
    ];
    
    /**
     * Generate a new OTP for the given email
     * 
     * @param string $email
     * @return string
     */
    public static function generateOtp(string $email): string
    {
        // Invalidate any existing OTPs for this email
        self::where('email', $email)->update(['used' => true]);
        
        // Generate a 6-digit OTP
        $otp = rand(100000, 999999);
        
        // Set expiry to 5 minutes from now
        $expiresAt = Carbon::now()->addMinutes(5);
        
        // Create the OTP record
        self::create([
            'email' => $email,
            'otp' => $otp,
            'expires_at' => $expiresAt,
            'used' => false
        ]);
        
        return $otp;
    }
    
    /**
     * Verify if the given OTP is valid for the email
     * 
     * @param string $email
     * @param string $otp
     * @return bool
     */
    public static function verifyOtp(string $email, string $otp): bool
    {
        $record = self::where('email', $email)
            ->where('otp', $otp)
            ->where('used', false)
            ->where('expires_at', '>', Carbon::now())
            ->first();
            
        if ($record) {
            // Mark as used
            $record->update(['used' => true]);
            return true;
        }
        
        return false;
    }
    
    /**
     * Check if the OTP has expired
     * 
     * @return bool
     */
    public function isExpired(): bool
    {
        return $this->expires_at < Carbon::now();
    }
}
